"""Dashboard service for Claude Monitor web interface.

This service provides a clean interface for web routes to access dashboard data
by wrapping existing parsers and analyzers. It reuses all existing business logic
while returning data as dicts/dataclasses suitable for web rendering.
"""

from datetime import datetime
from pathlib import Path
from typing import Optional, Dict, List, Any

from ...utils.paths import get_claude_paths, ClaudeDataPaths
from ...utils.time_filter import TimeFilter
from ...parsers.history import HistoryParser
from ...parsers.sessions import SessionParser
from ...parsers.debug import DebugLogParser
from ...parsers.files import FileHistoryParser
from ...parsers.tools import ToolUsageParser
from ...parsers.skills import SkillsParser, ConfigurationParser
from ...parsers.configuration_scanner import ConfigurationScanner
from ...analyzers.usage import UsageAnalyzer, UsageSummary
from ...analyzers.tokens import TokenAnalyzer, TokenSummary, get_model_display_name
from ...analyzers.integrations import IntegrationAnalyzer, IntegrationSummary
from ...analyzers.features import FeaturesAnalyzer, FeaturesSummary
from ...analyzers.configuration import ConfigurationAnalyzer
from ...analyzers.project_analyzer import ProjectAnalyzer


class DashboardService:
    """Service for providing dashboard data to web routes.

    Initializes all parsers and analyzers, then provides methods to retrieve
    aggregated data suitable for web rendering.
    """

    def __init__(self, claude_data_paths: Optional[ClaudeDataPaths] = None):
        """
        Initialize dashboard service with parsers and analyzers.

        Args:
            claude_data_paths: Optional ClaudeDataPaths instance. If not provided,
                             uses default Claude data directory.
        """
        # Use provided paths or get default
        if claude_data_paths is None:
            claude_data_paths = get_claude_paths()

        self.paths = claude_data_paths

        # Initialize parsers
        self._history_parser = HistoryParser(self.paths.history_file)
        self._session_parser = SessionParser(self.paths.get_project_session_files())
        self._file_parser = FileHistoryParser(
            self.paths.file_history_dir,
            self.paths.get_project_session_files()
        )
        self._tool_parser = ToolUsageParser(self.paths.get_project_session_files())
        self._skills_parser = SkillsParser(self.paths.base_dir / "skills")
        self._config_parser = ConfigurationParser(self.paths.base_dir)

        # Build project map for debug logs
        project_map = self._build_project_map()
        self._debug_parser = DebugLogParser(self.paths.get_debug_log_files(), project_map)

        # Initialize analyzers (no time filter by default)
        self._usage_analyzer = UsageAnalyzer(
            self._history_parser,
            self._session_parser,
            time_filter=None
        )
        self._token_analyzer = TokenAnalyzer(
            self._session_parser,
            time_filter=None
        )
        self._integration_analyzer = IntegrationAnalyzer(self._debug_parser)
        self._features_analyzer = FeaturesAnalyzer(
            self._tool_parser,
            self._skills_parser,
            self._debug_parser,
            self._config_parser,
            time_filter=None
        )

        # Initialize configuration scanner and analyzer
        self._config_scanner = ConfigurationScanner(self._config_parser)
        self._configuration_analyzer = ConfigurationAnalyzer(self._config_scanner)

        # Initialize project analyzer
        self._project_analyzer = ProjectAnalyzer(
            self._session_parser,
            self._tool_parser,
            self._skills_parser,
            self._config_scanner,
        )

    def _build_project_map(self) -> Dict[str, str]:
        """
        Build a map of session IDs to project names.

        Returns:
            Dict mapping session_id to project_name
        """
        project_paths = {}
        if self.paths.projects_dir.exists():
            all_dirs = sorted(
                [d for d in self.paths.projects_dir.iterdir() if d.is_dir()],
                key=lambda x: len(x.name),
                reverse=True
            )

            for project_dir in all_dirs:
                encoded_name = project_dir.name
                for session_file in project_dir.glob("*.jsonl"):
                    session_id = session_file.stem
                    if session_id not in project_paths or len(encoded_name) > len(project_paths[session_id]):
                        # Extract project name from encoded path
                        if encoded_name.startswith('-Users-allannapier-code-'):
                            project_name = encoded_name.replace('-Users-allannapier-code-', '', 1)
                        elif encoded_name.startswith('-'):
                            project_name = encoded_name[1:]
                        else:
                            project_name = encoded_name
                        project_paths[session_id] = project_name

        return project_paths

    def _create_time_filtered_service(self, time_filter: TimeFilter) -> 'DashboardService':
        """
        Create a new service instance with time filter applied.

        Args:
            time_filter: TimeFilter to apply

        Returns:
            DashboardService instance with time-filtered analyzers
        """
        service = DashboardService.__new__(DashboardService)
        service.paths = self.paths
        service._history_parser = self._history_parser
        service._session_parser = self._session_parser
        service._file_parser = self._file_parser
        service._tool_parser = self._tool_parser
        service._skills_parser = self._skills_parser
        service._config_parser = self._config_parser
        service._debug_parser = self._debug_parser

        # Create time-filtered analyzers
        service._usage_analyzer = UsageAnalyzer(
            self._history_parser,
            self._session_parser,
            time_filter=time_filter
        )
        service._token_analyzer = TokenAnalyzer(
            self._session_parser,
            time_filter=time_filter
        )
        service._integration_analyzer = IntegrationAnalyzer(self._debug_parser)
        service._features_analyzer = FeaturesAnalyzer(
            self._tool_parser,
            self._skills_parser,
            self._debug_parser,
            self._config_parser,
            time_filter=time_filter
        )

        return service

    # ========== Public Methods for Usage Data ==========

    def get_usage_summary(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get usage summary statistics.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict with usage statistics suitable for web rendering
        """
        analyzer = self._usage_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._usage_analyzer

        summary: UsageSummary = analyzer.get_summary()

        # Get actual project count from project breakdown
        project_data = self.get_project_breakdown(time_filter)
        actual_project_count = project_data.get('total_projects', 0)

        return {
            'total_commands': summary.total_commands,
            'total_sessions': summary.total_sessions,
            'total_messages': summary.total_messages,
            'total_projects': actual_project_count,  # Use actual count from projects
            'earliest_activity': summary.earliest_activity.isoformat() if summary.earliest_activity else None,
            'latest_activity': summary.latest_activity.isoformat() if summary.latest_activity else None,
            'time_range_description': summary.time_range_description,
            'date_range_days': summary.date_range_days,
            'avg_commands_per_day': round(summary.avg_commands_per_day, 2) if summary.avg_commands_per_day else None,
            'avg_sessions_per_day': round(summary.avg_sessions_per_day, 2) if summary.avg_sessions_per_day else None,
        }

    def get_project_breakdown(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get per-project activity breakdown.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict mapping project paths to activity metrics
        """
        analyzer = self._usage_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._usage_analyzer

        breakdown = analyzer.get_project_breakdown()

        # Get token breakdown for cost and token data
        token_analyzer = self._token_analyzer
        if time_filter:
            token_analyzer = self._create_time_filtered_service(time_filter)._token_analyzer
        token_breakdown = token_analyzer.get_project_breakdown()

        # Convert to list of dicts with field names expected by template
        projects_data = []
        total_cost = 0.0
        most_active_project = None
        max_commands = 0

        for project_path, project_data in breakdown.items():
            # Get token data for this project
            token_summary = token_breakdown.get(project_path)
            total_tokens = 0
            cost = 0.0

            if token_summary:
                total_tokens = (
                    token_summary.total_tokens.input_tokens +
                    token_summary.total_tokens.output_tokens +
                    token_summary.total_tokens.cache_creation_input_tokens +
                    token_summary.total_tokens.cache_read_input_tokens
                )
                cost = token_summary.total_cost
                total_cost += cost

            # Determine most active project (by commands)
            if project_data['commands'] > max_commands:
                max_commands = project_data['commands']
                most_active_project = project_data['name']

            projects_data.append({
                'path': project_path,
                'name': project_data['name'],
                'full_path': project_data['full_path'],
                # Field names expected by template
                'command_count': project_data['commands'],
                'session_count': project_data['sessions'],
                'message_count': project_data['messages'],
                'total_tokens': total_tokens,
                'total_cost': cost,
                # Keep original names for backward compatibility
                'commands': project_data['commands'],
                'sessions': project_data['sessions'],
                'messages': project_data['messages'],
            })

        # Sort by command count (descending)
        projects_data.sort(key=lambda x: x['command_count'], reverse=True)

        # Calculate average commands per project
        avg_commands = sum(p['command_count'] for p in projects_data) / len(projects_data) if projects_data else 0

        return {
            'projects': projects_data,
            'total_projects': len(projects_data),
            'total_cost': round(total_cost, 2),
            'avg_commands_per_project': round(avg_commands, 1),
            'most_active_project': most_active_project or 'N/A'
        }

    def get_project_breakdown_by_model(
        self,
        model_id: str,
        time_filter: Optional[TimeFilter] = None
    ) -> Dict[str, Any]:
        """
        Get per-project activity breakdown filtered by a specific model.

        Args:
            model_id: Model ID to filter by
            time_filter: Optional time filter to apply

        Returns:
            Dict with project data filtered to only show usage of the specified model
        """
        analyzer = self._usage_analyzer
        token_analyzer = self._token_analyzer
        if time_filter:
            filtered_service = self._create_time_filtered_service(time_filter)
            analyzer = filtered_service._usage_analyzer
            token_analyzer = filtered_service._token_analyzer

        # Get base project breakdown for structure (commands, sessions, etc.)
        base_breakdown = analyzer.get_project_breakdown()

        # Get model-by-project breakdown for token/cost data
        model_by_project = token_analyzer.get_model_by_project_breakdown()

        projects_data = []
        total_cost = 0.0
        most_active_project = None
        max_cost = 0.0

        for project_path, project_data in base_breakdown.items():
            # Get model-specific data for this project
            project_models = model_by_project.get(project_path, {})

            if model_id in project_models:
                tokens, cost = project_models[model_id]
                total_tokens = (
                    tokens.input_tokens +
                    tokens.output_tokens +
                    tokens.cache_creation_input_tokens +
                    tokens.cache_read_input_tokens
                )
                total_cost += cost

                # Track most active by cost for model filter
                if cost > max_cost:
                    max_cost = cost
                    most_active_project = project_data['name']

                projects_data.append({
                    'path': project_path,
                    'name': project_data['name'],
                    'full_path': project_data['full_path'],
                    'command_count': project_data['commands'],
                    'session_count': project_data['sessions'],
                    'message_count': project_data['messages'],
                    'total_tokens': total_tokens,
                    'total_cost': cost,
                    'commands': project_data['commands'],
                    'sessions': project_data['sessions'],
                    'messages': project_data['messages'],
                })

        # Sort by cost (descending) for model filter
        projects_data.sort(key=lambda x: x['total_cost'], reverse=True)

        avg_commands = sum(p['command_count'] for p in projects_data) / len(projects_data) if projects_data else 0

        return {
            'projects': projects_data,
            'total_projects': len(projects_data),
            'total_cost': round(total_cost, 2),
            'avg_commands_per_project': round(avg_commands, 1),
            'most_active_project': most_active_project or 'N/A'
        }

    # ========== Public Methods for Token Data ==========

    def get_token_summary(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get token usage summary with cost calculations.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict with token statistics and costs (flattened for template use)
        """
        analyzer = self._token_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._token_analyzer

        summary: TokenSummary = analyzer.get_summary()

        return {
            # Flat structure for templates
            'input_tokens': summary.total_tokens.input_tokens,
            'output_tokens': summary.total_tokens.output_tokens,
            'cache_creation_tokens': summary.total_tokens.cache_creation_input_tokens,
            'cache_read_tokens': summary.total_tokens.cache_read_input_tokens,
            'input_cost': round(summary.cost_breakdown.input_cost, 2),
            'output_cost': round(summary.cost_breakdown.output_cost, 2),
            'cache_creation_cost': round(summary.cost_breakdown.cache_write_cost, 2),
            'cache_read_cost': round(summary.cost_breakdown.cache_read_cost, 2),
            'total_cost': round(summary.cost_breakdown.total_cost, 2),
            'cache_savings': round(summary.cost_breakdown.cache_savings, 2),
            'cache_hit_rate': round(summary.cache_efficiency_pct, 1),
        }

    def get_model_breakdown(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get per-model token usage and costs.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict mapping model names to token usage and costs
        """
        analyzer = self._token_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._token_analyzer

        breakdown = analyzer.get_model_breakdown()

        models_data = []
        total_cost = sum(cost for _, cost in breakdown.values())

        for model_id, (tokens, cost) in breakdown.items():
            total_tokens = (
                tokens.input_tokens +
                tokens.output_tokens +
                tokens.cache_creation_input_tokens +
                tokens.cache_read_input_tokens
            )
            models_data.append({
                'model_id': model_id,
                'model_name': get_model_display_name(model_id),
                'input_tokens': tokens.input_tokens,
                'output_tokens': tokens.output_tokens,
                'cache_creation_tokens': tokens.cache_creation_input_tokens,
                'cache_read_tokens': tokens.cache_read_input_tokens,
                'total_tokens': total_tokens,
                'cost': round(cost, 4),
                'cost_percentage': round((cost / total_cost * 100) if total_cost > 0 else 0, 1),
            })

        return {
            'models': models_data,
            'total_models': len(models_data),
            'total_cost': round(total_cost, 2),
        }

    def get_available_models(self, time_filter: Optional[TimeFilter] = None) -> List[Dict[str, str]]:
        """
        Get list of available models for filtering.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            List of dicts with model id and display name
        """
        breakdown = self.get_model_breakdown(time_filter)
        return [
            {'id': model['model_id'], 'name': model['model_name']}
            for model in breakdown['models']
        ]

    def get_daily_token_trend(
        self,
        days: int = 7,
        time_filter: Optional[TimeFilter] = None
    ) -> Dict[str, Any]:
        """
        Get daily token usage for trend chart.

        Args:
            days: Number of days to include (default 7). Note: when time_filter
                  is provided, days is calculated from the filter's start_time
                  and this parameter is ignored.
            time_filter: Optional time filter - if provided, overrides days parameter

        Returns:
            Dict with labels and data arrays for charting
        """
        # If time filter is provided, calculate days from it (overrides days param)
        if time_filter and time_filter.start_time:
            now = datetime.now()
            delta = now - time_filter.start_time
            days = max(1, delta.days + 1)  # Include current partial day, at least 1 day

        daily_stats = self._session_parser.get_daily_stats(days=days, time_filter=time_filter)

        labels = []
        data = []
        today = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

        for date_str in sorted(daily_stats.keys()):
            stats = daily_stats[date_str]
            # Calculate total tokens for this day
            total = (
                stats.total_tokens.input_tokens +
                stats.total_tokens.output_tokens +
                stats.total_tokens.cache_creation_input_tokens +
                stats.total_tokens.cache_read_input_tokens
            )
            data.append(total)

            # Create human-readable label based on how many days we're showing
            date = datetime.strptime(date_str, '%Y-%m-%d')
            days_ago = (today - date).days

            if days <= 7:
                # For week or less, show relative labels
                if days_ago == 0:
                    labels.append('Today')
                elif days_ago == 1:
                    labels.append('Yesterday')
                else:
                    labels.append(f'{days_ago}d ago')
            else:
                # For longer periods, show date labels (Dec 15 format)
                labels.append(date.strftime('%b %d'))

        return {
            'labels': labels,
            'data': data,
        }

    def get_daily_cost_trend(
        self,
        days: int = 7,
        time_filter: Optional[TimeFilter] = None
    ) -> Dict[str, Any]:
        """
        Get daily cost trend for charts.

        Args:
            days: Number of days to include (default 7)
            time_filter: Optional time filter - if provided, overrides days parameter

        Returns:
            Dict with labels and datasets for Chart.js
        """
        # If time filter is provided, calculate days from it
        if time_filter and time_filter.start_time:
            now = datetime.now()
            delta = now - time_filter.start_time
            days = max(1, delta.days + 1)

        daily_costs = self._session_parser.get_daily_cost_trend(days=days, time_filter=time_filter)

        labels = []
        total_costs = []
        input_costs = []
        output_costs = []
        cache_costs = []

        today = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

        for date_str in sorted(daily_costs.keys()):
            costs = daily_costs[date_str]
            total_costs.append(costs['total_cost'])
            input_costs.append(costs['input_cost'])
            output_costs.append(costs['output_cost'])
            cache_costs.append(costs['cache_write_cost'] + costs['cache_read_cost'])

            # Create human-readable label based on how many days we're showing
            date = datetime.strptime(date_str, '%Y-%m-%d')
            days_ago = (today - date).days

            if days <= 7:
                # For week or less, show relative labels
                if days_ago == 0:
                    labels.append('Today')
                elif days_ago == 1:
                    labels.append('Yesterday')
                else:
                    labels.append(f'{days_ago}d ago')
            else:
                # For longer periods, show date labels (Dec 15 format)
                labels.append(date.strftime('%b %d'))

        return {
            'labels': labels,
            'datasets': {
                'total': total_costs,
                'input': input_costs,
                'output': output_costs,
                'cache': cache_costs,
            }
        }

    def get_project_cost_trend(
        self,
        days: int = 7,
        time_filter: Optional[TimeFilter] = None,
        max_projects: int = 8
    ) -> Dict[str, Any]:
        """
        Get daily cost trend per project for charts.

        Args:
            days: Number of days to include (default 7)
            time_filter: Optional time filter - if provided, overrides days parameter
            max_projects: Maximum number of projects to include

        Returns:
            Dict with labels and datasets for Chart.js line chart
        """
        # If time filter is provided, calculate days from it
        if time_filter and time_filter.start_time:
            now = datetime.now()
            delta = now - time_filter.start_time
            days = max(1, delta.days + 1)

        project_daily_stats = self._session_parser.get_project_daily_stats(
            days=days,
            time_filter=time_filter,
            max_projects=max_projects
        )

        # Get date labels from the first project
        labels = []
        if project_daily_stats:
            first_project = list(project_daily_stats.values())[0]
            today = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

            for date_str in sorted(first_project.keys()):
                # Create human-readable label
                date = datetime.strptime(date_str, '%Y-%m-%d')
                days_ago = (today - date).days

                if days <= 7:
                    if days_ago == 0:
                        labels.append('Today')
                    elif days_ago == 1:
                        labels.append('Yesterday')
                    else:
                        labels.append(f'{days_ago}d ago')
                else:
                    labels.append(date.strftime('%b %d'))

        # Build datasets for each project
        datasets = []
        colors = [
            '#0770E3',  # Blue
            '#34D399',  # Green
            '#F59E0B',  # Amber
            '#8B5CF6',  # Purple
            '#EC4899',  # Pink
            '#14B8A6',  # Teal
            '#F97316',  # Orange
            '#6366F1',  # Indigo
        ]

        for idx, (project_name, daily_data) in enumerate(project_daily_stats.items()):
            color = colors[idx % len(colors)]
            data = [daily_data[date]['total_cost'] for date in sorted(daily_data.keys())]

            datasets.append({
                'label': project_name,
                'data': data,
                'borderColor': color,
                'color': color,
            })

        return {
            'labels': labels,
            'datasets': datasets,
        }

    def get_token_summary_by_model(
        self,
        model_id: str,
        time_filter: Optional[TimeFilter] = None
    ) -> Dict[str, Any]:
        """
        Get token usage summary filtered by a specific model.

        Args:
            model_id: Model ID to filter by
            time_filter: Optional time filter to apply

        Returns:
            Dict with token statistics for the specified model
        """
        analyzer = self._token_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._token_analyzer

        breakdown = analyzer.get_model_breakdown()

        if model_id not in breakdown:
            # Return empty summary if model not found
            return {
                'input_tokens': 0,
                'output_tokens': 0,
                'cache_creation_tokens': 0,
                'cache_read_tokens': 0,
                'input_cost': 0.0,
                'output_cost': 0.0,
                'cache_creation_cost': 0.0,
                'cache_read_cost': 0.0,
                'total_cost': 0.0,
                'cache_savings': 0.0,
                'cache_hit_rate': 0.0,
            }

        tokens, total_cost = breakdown[model_id]
        cost_breakdown = analyzer.calculate_cost(tokens, model_id)

        # Calculate cache efficiency for this model
        total_input = tokens.input_tokens + tokens.cache_creation_input_tokens + tokens.cache_read_input_tokens
        cache_hit_rate = (tokens.cache_read_input_tokens / total_input * 100) if total_input > 0 else 0

        return {
            'input_tokens': tokens.input_tokens,
            'output_tokens': tokens.output_tokens,
            'cache_creation_tokens': tokens.cache_creation_input_tokens,
            'cache_read_tokens': tokens.cache_read_input_tokens,
            'input_cost': round(cost_breakdown.input_cost, 2),
            'output_cost': round(cost_breakdown.output_cost, 2),
            'cache_creation_cost': round(cost_breakdown.cache_write_cost, 2),
            'cache_read_cost': round(cost_breakdown.cache_read_cost, 2),
            'total_cost': round(cost_breakdown.total_cost, 2),
            'cache_savings': round(cost_breakdown.cache_savings, 2),
            'cache_hit_rate': round(cache_hit_rate, 1),
        }

    def get_project_token_breakdown(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get token usage breakdown per project.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict mapping project names to token summaries
        """
        analyzer = self._token_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._token_analyzer

        breakdown = analyzer.get_project_breakdown()

        projects_data = []
        total_cost = 0.0

        for project_path, summary in breakdown.items():
            project_data = {
                'project_path': project_path,
                'total_tokens': {
                    'input_tokens': summary.total_tokens.input_tokens,
                    'output_tokens': summary.total_tokens.output_tokens,
                    'cache_creation_input_tokens': summary.total_tokens.cache_creation_input_tokens,
                    'cache_read_input_tokens': summary.total_tokens.cache_read_input_tokens,
                },
                'cost_breakdown': {
                    'input_cost': round(summary.cost_breakdown.input_cost, 4),
                    'output_cost': round(summary.cost_breakdown.output_cost, 4),
                    'cache_write_cost': round(summary.cost_breakdown.cache_write_cost, 4),
                    'cache_read_cost': round(summary.cost_breakdown.cache_read_cost, 4),
                    'total_cost': round(summary.cost_breakdown.total_cost, 4),
                },
                'cache_efficiency_pct': round(summary.cache_efficiency_pct, 2),
            }
            projects_data.append(project_data)
            total_cost += summary.total_cost

        return {
            'projects': projects_data,
            'total_projects': len(projects_data),
            'total_cost': round(total_cost, 4),
        }

    # ========== Public Methods for Integration Data ==========

    def get_integration_summary(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get MCP integration usage summary.

        Args:
            time_filter: Optional time filter (not used by integration analyzer)

        Returns:
            Dict with integration statistics
        """
        summary: IntegrationSummary = self._integration_analyzer.get_summary()

        top_servers_data = [
            {
                'server_name': name,
                'tool_call_count': stats.tool_call_count,
                'connection_count': stats.connection_count,
                'error_count': stats.error_count,
                'error_rate': round(
                    (stats.error_count / stats.tool_call_count * 100)
                    if stats.tool_call_count > 0 else 0,
                    2
                ),
            }
            for name, stats in summary.top_servers
        ]

        return {
            'total_servers': summary.total_servers,
            'total_tool_calls': summary.total_tool_calls,
            'total_connections': summary.total_connections,
            'total_errors': summary.total_errors,
            'error_rate': round(
                (summary.total_errors / summary.total_tool_calls * 100)
                if summary.total_tool_calls > 0 else 0,
                2
            ),
            'has_integrations': summary.has_integrations,
            'top_servers': top_servers_data,
        }

    def get_server_details(self) -> Dict[str, Any]:
        """
        Get detailed statistics for all MCP servers.

        Returns:
            Dict mapping server names to detailed statistics
        """
        all_servers = self._integration_analyzer.get_server_details()

        servers_data = {}
        for server_name, stats in all_servers.items():
            servers_data[server_name] = {
                'tool_call_count': stats.tool_call_count,
                'connection_count': stats.connection_count,
                'error_count': stats.error_count,
                'error_rate': round(
                    (stats.error_count / stats.tool_call_count * 100)
                    if stats.tool_call_count > 0 else 0,
                    2
                ),
            }

        return {
            'servers': servers_data,
            'total_servers': len(servers_data),
        }

    # ========== Public Methods for Features Data ==========

    def get_features_summary(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get comprehensive features usage summary.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict with all feature data
        """
        analyzer = self._features_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._features_analyzer

        summary: FeaturesSummary = analyzer.get_summary()

        return {
            'subagents': {
                'total_calls': summary.total_subagent_calls,
                'unique_used': summary.unique_subagents_used,
                'stats': {
                    name: {
                        'invocation_count': stats.invocation_count,
                        'total_tokens': stats.total_tokens,
                        'session_count': stats.session_count,
                    }
                    for name, stats in summary.subagent_stats.items()
                },
            },
            'skills': {
                'total_installed': summary.total_skills,
                'installed': [
                    {
                        'name': skill.name,
                        'description': skill.description,
                        'has_skills_md': skill.has_skills_md,
                    }
                    for skill in summary.installed_skills
                ],
            },
            'mcps': {
                'total_servers': summary.total_mcp_servers,
                'enabled': summary.enabled_mcps,
                'stats': {
                    name: {
                        'tool_call_count': stats.tool_call_count,
                        'connection_count': stats.connection_count,
                        'error_count': stats.error_count,
                    }
                    for name, stats in summary.mcp_stats.items()
                },
            },
            'tools': {
                'total_calls': summary.total_tool_calls,
                'stats': {
                    name: {
                        'invocation_count': stats.invocation_count,
                        'total_tokens': stats.total_tokens,
                        'session_count': stats.session_count,
                    }
                    for name, stats in summary.tool_stats.items()
                },
            },
            'configuration': {
                'always_thinking_enabled': summary.always_thinking_enabled,
                'enabled_plugins': summary.enabled_plugins,
            },
        }

    def get_top_tools(self, limit: int = 10, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get most frequently used tools.

        Args:
            limit: Maximum number of tools to return
            time_filter: Optional time filter to apply

        Returns:
            List of top tools with usage stats
        """
        from ...analyzers.tokens import DEFAULT_PRICING

        analyzer = self._features_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._features_analyzer

        top_tools = analyzer.get_top_tools(limit=limit)

        total_tokens = 0
        total_cost = 0.0
        total_calls = 0
        tools_data = []

        for name, stats in top_tools:
            # Calculate cost using default Sonnet pricing
            input_cost = (stats.total_input_tokens / 1_000_000) * DEFAULT_PRICING['input_per_mtok']
            output_cost = (stats.total_output_tokens / 1_000_000) * DEFAULT_PRICING['output_per_mtok']
            cache_read_cost = (stats.total_cache_read_tokens / 1_000_000) * DEFAULT_PRICING['cache_read_per_mtok']
            cache_write_cost = (stats.total_cache_write_tokens / 1_000_000) * DEFAULT_PRICING['cache_write_per_mtok']
            tool_cost = input_cost + output_cost + cache_read_cost + cache_write_cost

            total_tokens += stats.total_tokens
            total_cost += tool_cost
            total_calls += stats.invocation_count

            tools_data.append({
                'tool_name': name,
                'invocation_count': stats.invocation_count,
                'total_tokens': stats.total_tokens,
                'input_tokens': stats.total_input_tokens,
                'output_tokens': stats.total_output_tokens,
                'total_cost': round(tool_cost, 4),
                'avg_tokens_per_call': round(
                    stats.total_tokens / stats.invocation_count
                    if stats.invocation_count > 0 else 0,
                    1
                ),
                'session_count': stats.session_count,
            })

        return {
            'tools': tools_data,
            'total_shown': len(tools_data),
            'total_tokens': total_tokens,
            'total_calls': total_calls,
            'total_cost': round(total_cost, 2),
        }

    def get_top_subagents(self, limit: int = 10, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get most frequently used sub-agents.

        Args:
            limit: Maximum number of sub-agents to return
            time_filter: Optional time filter to apply

        Returns:
            List of top sub-agents with usage stats
        """
        analyzer = self._features_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._features_analyzer

        top_subagents = analyzer.get_top_subagents(limit=limit)

        subagents_data = [
            {
                'type': name,
                'invocation_count': stats.invocation_count,
                'success_count': stats.success_count,
                'error_count': stats.error_count,
                'success_rate': round(
                    (stats.success_count / stats.invocation_count * 100)
                    if stats.invocation_count > 0 else 0,
                    2
                ),
            }
            for name, stats in top_subagents
        ]

        return {
            'subagents': subagents_data,
            'total_shown': len(subagents_data),
        }

    def get_mcp_integrations(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get MCP server integration statistics.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict with MCP server usage data
        """
        from ...analyzers.tokens import DEFAULT_PRICING

        analyzer = self._features_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._features_analyzer

        # Get all tool stats from the tool parser
        all_tools = analyzer.tool_parser.get_tool_stats(time_filter=time_filter)

        # Filter for MCP tools (tools that start with mcp__)
        mcp_tools = {name: stats for name, stats in all_tools.items() if name.startswith('mcp__')}

        # Group by MCP server
        servers = {}
        for tool_name, stats in mcp_tools.items():
            # Extract server name from tool name: mcp__server_name__function_name
            parts = tool_name.split('__')
            if len(parts) >= 2:
                server_name = parts[1]

                if server_name not in servers:
                    servers[server_name] = {
                        'server_name': server_name,
                        'tool_count': 0,
                        'total_calls': 0,
                        'total_tokens': 0,
                        'input_tokens': 0,
                        'output_tokens': 0,
                        'cache_read_tokens': 0,
                        'cache_write_tokens': 0,
                        'total_cost': 0.0,
                        'tools': []
                    }

                servers[server_name]['tool_count'] += 1
                servers[server_name]['total_calls'] += stats.invocation_count
                servers[server_name]['total_tokens'] += stats.total_tokens
                servers[server_name]['input_tokens'] += stats.total_input_tokens
                servers[server_name]['output_tokens'] += stats.total_output_tokens
                servers[server_name]['cache_read_tokens'] += stats.total_cache_read_tokens
                servers[server_name]['cache_write_tokens'] += stats.total_cache_write_tokens
                servers[server_name]['tools'].append({
                    'tool_name': tool_name,
                    'invocation_count': stats.invocation_count,
                    'total_tokens': stats.total_tokens
                })

        # Calculate costs per server using default pricing (Sonnet 4.5)
        # Note: Tool-level token tracking doesn't include model info, so we use default rates
        for server in servers.values():
            input_cost = (server['input_tokens'] / 1_000_000) * DEFAULT_PRICING['input_per_mtok']
            output_cost = (server['output_tokens'] / 1_000_000) * DEFAULT_PRICING['output_per_mtok']
            cache_read_cost = (server['cache_read_tokens'] / 1_000_000) * DEFAULT_PRICING['cache_read_per_mtok']
            cache_write_cost = (server['cache_write_tokens'] / 1_000_000) * DEFAULT_PRICING['cache_write_per_mtok']
            server['total_cost'] = round(input_cost + output_cost + cache_read_cost + cache_write_cost, 4)

        # Sort servers by total calls
        sorted_servers = sorted(servers.values(), key=lambda x: x['total_calls'], reverse=True)

        # Calculate totals
        total_mcp_servers = len(servers)
        total_mcp_tools = len(mcp_tools)
        total_mcp_calls = sum(stats.invocation_count for stats in mcp_tools.values())
        total_mcp_tokens = sum(s['total_tokens'] for s in servers.values())
        total_mcp_cost = sum(s['total_cost'] for s in servers.values())
        most_used_server = sorted_servers[0]['server_name'] if sorted_servers else 'None'

        return {
            'total_servers': total_mcp_servers,
            'total_tools': total_mcp_tools,
            'total_calls': total_mcp_calls,
            'total_tokens': total_mcp_tokens,
            'total_cost': round(total_mcp_cost, 2),
            'most_used_server': most_used_server,
            'servers': sorted_servers
        }

    def get_file_statistics(self, limit: int = 20, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get file operation statistics from Read, Write, and Edit tool usage.

        Args:
            limit: Maximum number of files to return
            time_filter: Optional time filter to apply

        Returns:
            Dict with file operation statistics
        """
        analyzer = self._features_analyzer
        if time_filter:
            analyzer = self._create_time_filtered_service(time_filter)._features_analyzer

        # Get all tool invocations
        file_ops = {}

        for invocation in analyzer.tool_parser.parse_all(time_filter=time_filter):
            if invocation.tool_name in ['Read', 'Write', 'Edit']:
                file_path = invocation.input_params.get('file_path')
                if file_path:
                    if file_path not in file_ops:
                        file_ops[file_path] = {
                            'file_path': file_path,
                            'read_count': 0,
                            'write_count': 0,
                            'edit_count': 0,
                            'total_operations': 0,
                            'projects': set()
                        }

                    tool_type = invocation.tool_name.lower()
                    file_ops[file_path][f'{tool_type}_count'] += 1
                    file_ops[file_path]['total_operations'] += 1

                    if invocation.project:
                        file_ops[file_path]['projects'].add(invocation.project)

        # Convert sets to counts and create list
        files_list = []
        for file_path, ops in file_ops.items():
            ops['project_count'] = len(ops['projects'])
            ops.pop('projects')  # Remove set before returning
            files_list.append(ops)

        # Sort by total operations
        sorted_files = sorted(files_list, key=lambda x: x['total_operations'], reverse=True)[:limit]

        # Calculate statistics
        total_files = len(file_ops)
        total_operations = sum(f['total_operations'] for f in files_list)
        total_edits = sum(f['edit_count'] for f in files_list)
        total_reads = sum(f['read_count'] for f in files_list)
        total_writes = sum(f['write_count'] for f in files_list)

        most_edited_file = sorted_files[0]['file_path'] if sorted_files else 'N/A'
        if most_edited_file != 'N/A':
            # Shorten path for display
            import os
            most_edited_file = os.path.basename(most_edited_file)

        avg_edits_per_file = total_edits / total_files if total_files > 0 else 0

        return {
            'total_files': total_files,
            'most_edited_file': most_edited_file,
            'avg_edits_per_file': round(avg_edits_per_file, 1),
            'total_operations': total_operations,
            'total_edits': total_edits,
            'total_reads': total_reads,
            'total_writes': total_writes,
            'file_operations': sorted_files
        }

    # ========== Public Methods for Full Dashboard ==========

    def get_full_dashboard(self, time_filter: Optional[TimeFilter] = None) -> Dict[str, Any]:
        """
        Get complete dashboard data combining all metrics.

        Args:
            time_filter: Optional time filter to apply

        Returns:
            Dict with all dashboard data
        """
        return {
            'usage': self.get_usage_summary(time_filter=time_filter),
            'project_breakdown': self.get_project_breakdown(time_filter=time_filter),
            'tokens': self.get_token_summary(time_filter=time_filter),
            'models': self.get_model_breakdown(time_filter=time_filter),
            'integrations': self.get_integration_summary(time_filter=time_filter),
            'features': self.get_features_summary(time_filter=time_filter),
            'top_tools': self.get_top_tools(limit=5, time_filter=time_filter),
            'top_subagents': self.get_top_subagents(limit=5, time_filter=time_filter),
        }

    # ========== Configuration Methods ==========

    def get_discovered_repositories(self) -> List[Dict]:
        """
        Get all discovered repositories with .claude/ directories.

        Returns:
            List of repository dicts
        """
        repos = self._config_scanner.scan_for_repositories()
        return [repo.to_dict() for repo in repos]

    def get_configuration_features(self, repo_path: str) -> Dict[str, Any]:
        """
        Get all configuration features for a repository.

        Args:
            repo_path: Path to repository as string

        Returns:
            Dict with all features categorized
        """
        path = Path(repo_path)
        return self._configuration_analyzer.get_feature_breakdown(path)

    def get_feature_detail(
        self, repo_path: str, feature_type: str, feature_id: str
    ) -> Dict[str, Any]:
        """
        Get detailed information about a specific feature.

        Args:
            repo_path: Path to repository as string
            feature_type: Type of feature (skill, command, etc.)
            feature_id: ID/name of the feature

        Returns:
            Dict with feature details
        """
        path = Path(repo_path)
        return self._configuration_analyzer.get_feature_detail(
            feature_type, feature_id, path
        )

    def get_inheritance_chain(
        self, repo_path: str, feature_type: str, feature_id: str
    ) -> List[Dict]:
        """
        Get inheritance chain for a feature.

        Args:
            repo_path: Path to repository as string
            feature_type: Type of feature
            feature_id: ID/name of the feature

        Returns:
            List of inheritance levels
        """
        path = Path(repo_path)
        inheritance_tree = self._configuration_analyzer.get_inheritance_tree(
            feature_type, feature_id, path
        )
        return inheritance_tree.get('levels', [])

    # ========== Project Analysis Methods ==========

    def get_project_analysis(
        self,
        project_path: str,
        project_name: str,
        time_filter: Optional[TimeFilter] = None
    ) -> Dict[str, Any]:
        """
        Get optimization recommendations for a specific project.

        Args:
            project_path: Path to the project
            project_name: Display name of the project
            time_filter: Optional time filter for analysis period

        Returns:
            Dict with project analysis including recommendations and metrics
        """
        analysis = self._project_analyzer.analyze_project(
            project_path=project_path,
            project_name=project_name,
            time_filter=time_filter
        )
        return analysis.to_dict()

    def analyze_all_projects(
        self,
        time_filter: Optional[TimeFilter] = None
    ) -> Dict[str, Any]:
        """
        Analyze all projects for optimization opportunities.

        Args:
            time_filter: Optional time filter for analysis period

        Returns:
            Dict with all project analyses and summary
        """
        # Get all projects from project breakdown
        project_data = self.get_project_breakdown(time_filter)
        all_analyses = []

        for project in project_data.get('projects', []):
            # Skip projects with very low usage
            if project.get('command_count', 0) < 2:
                continue

            try:
                analysis = self._project_analyzer.analyze_project(
                    project_path=project.get('full_path', project.get('path', '')),
                    project_name=project.get('name', 'Unknown'),
                    time_filter=time_filter
                )
                all_analyses.append(analysis.to_dict())
            except Exception:
                # Skip projects that can't be analyzed
                continue

        # Sort by total recommendations (high severity first)
        all_analyses.sort(
            key=lambda x: (
                x.get('summary', {}).get('high_severity', 0),
                x.get('summary', {}).get('medium_severity', 0),
                x.get('summary', {}).get('low_severity', 0)
            ),
            reverse=True
        )

        # Calculate summary stats
        total_high = sum(a.get('summary', {}).get('high_severity', 0) for a in all_analyses)
        total_medium = sum(a.get('summary', {}).get('medium_severity', 0) for a in all_analyses)
        total_low = sum(a.get('summary', {}).get('low_severity', 0) for a in all_analyses)

        return {
            'projects': all_analyses,
            'total_projects_analyzed': len(all_analyses),
            'summary': {
                'total_high_severity': total_high,
                'total_medium_severity': total_medium,
                'total_low_severity': total_low,
                'total_recommendations': total_high + total_medium + total_low,
            }
        }



"""Models package for Claude Monitor."""

from datetime import datetime
from enum import Enum
from dataclasses import dataclass, field
from pathlib import Path
from typing import List, Optional, Dict, Any


class ConfigSource(Enum):
    """Source of a configuration feature."""
    USER = "user"
    PLUGIN = "plugin"
    PROJECT = "project"


@dataclass
class RepositoryConfig:
    """Repository configuration information."""
    path: Path
    name: str
    claude_dir: Path

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'path': str(self.path),
            'name': self.name,
            'claude_dir': str(self.claude_dir),
        }


@dataclass
class Skill:
    """Claude Code skill configuration."""
    name: str
    description: str
    source: ConfigSource
    path: Path
    content: str = ""
    version: Optional[str] = None
    agents: List[str] = field(default_factory=list)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'name': self.name,
            'description': self.description,
            'source': self.source.value,
            'path': str(self.path),
            'content': self.content,
            'version': self.version,
            'agents': self.agents,
        }


@dataclass
class MCP:
    """Model Context Protocol server configuration."""
    name: str
    command: str
    args: List[str]
    source: ConfigSource
    plugin_path: Optional[Path] = None
    env: Dict[str, str] = field(default_factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'name': self.name,
            'command': self.command,
            'args': self.args,
            'source': self.source.value,
            'plugin_path': str(self.plugin_path) if self.plugin_path else None,
            'env': self.env,
        }


@dataclass
class Command:
    """Slash command configuration."""
    name: str
    content: str
    source: ConfigSource
    path: Path
    description: str = ""

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'name': self.name,
            'content': self.content,
            'source': self.source.value,
            'path': str(self.path),
            'description': self.description,
        }


@dataclass
class Plugin:
    """Plugin configuration."""
    name: str
    version: str
    description: str
    install_path: Path
    source: ConfigSource = ConfigSource.PLUGIN
    author: Optional[str] = None
    license: Optional[str] = None
    keywords: List[str] = field(default_factory=list)
    enabled: bool = True
    features: Dict[str, int] = field(default_factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'name': self.name,
            'version': self.version,
            'description': self.description,
            'install_path': str(self.install_path),
            'source': self.source.value,
            'author': self.author,
            'license': self.license,
            'keywords': self.keywords,
            'enabled': self.enabled,
            'features': self.features,
        }


@dataclass
class Hook:
    """Hook configuration."""
    name: str
    hook_type: str
    triggers: List[str]
    matchers: Optional[List[str]]
    handler_type: str
    handler_path: Optional[Path]
    source: ConfigSource
    plugin_name: Optional[str] = None
    config: Dict[str, Any] = field(default_factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'name': self.name,
            'description': self.name,  # Use name as description for compatibility
            'hook_type': self.hook_type,
            'triggers': self.triggers,
            'matchers': self.matchers or [],
            'handler': str(self.handler_path) if self.handler_path else self.handler_type,
            'handler_type': self.handler_type,
            'handler_path': str(self.handler_path) if self.handler_path else None,
            'source': self.source.value,
            'plugin_name': self.plugin_name,
            'config': self.config,
        }


@dataclass
class Agent:
    """Agent configuration."""
    name: str
    description: str
    source: ConfigSource
    path: Path
    model: Optional[str] = None
    color: Optional[str] = None
    tools: List[str] = field(default_factory=list)
    instructions: str = ""

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'name': self.name,
            'description': self.description,
            'source': self.source.value,
            'path': str(self.path),
            'model': self.model,
            'color': self.color,
            'tools': self.tools,
            'instructions': self.instructions,
        }


@dataclass
class InheritanceLevel:
    """Single level in an inheritance chain."""
    source: ConfigSource
    path: Path
    feature: Any  # The actual feature object

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'source': self.source.value,
            'path': str(self.path),
            'feature': self.feature.to_dict() if hasattr(self.feature, 'to_dict') else str(self.feature),
        }


@dataclass
class InheritanceChain:
    """Full inheritance chain for a feature."""
    feature_type: str
    feature_name: str
    levels: List[InheritanceLevel] = field(default_factory=list)
    winner: Optional[ConfigSource] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'feature_type': self.feature_type,
            'feature_name': self.feature_name,
            'levels': [level.to_dict() for level in self.levels],
            'winner': self.winner.value if self.winner else None,
        }


@dataclass
class ConfigurationFeatures:
    """All configuration features for a repository."""
    skills: List[Skill] = field(default_factory=list)
    mcps: List[MCP] = field(default_factory=list)
    commands: List[Command] = field(default_factory=list)
    plugins: List[Plugin] = field(default_factory=list)
    hooks: List[Hook] = field(default_factory=list)
    agents: List[Agent] = field(default_factory=list)

    def get_feature_count(self) -> Dict[str, int]:
        """Get count of each feature type."""
        return {
            'skills': len(self.skills),
            'mcps': len(self.mcps),
            'commands': len(self.commands),
            'plugins': len(self.plugins),
            'hooks': len(self.hooks),
            'agents': len(self.agents),
        }


@dataclass
class Recommendation:
    """A single optimization recommendation for a project."""
    category: str  # 'mcp', 'model', 'config', 'cache', 'agent'
    severity: str  # 'high', 'medium', 'low', 'info'
    title: str
    description: str
    impact: str  # "Estimated savings: ~$X/month" or similar
    action_items: List[str] = field(default_factory=list)
    details: Dict[str, Any] = field(default_factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'category': self.category,
            'severity': self.severity,
            'title': self.title,
            'description': self.description,
            'impact': self.impact,
            'action_items': self.action_items,
            'details': self.details,
        }


@dataclass
class ProjectAnalysis:
    """Complete optimization analysis for a project."""
    project_path: str
    project_name: str
    recommendations: List[Recommendation]
    metrics: Dict[str, Any]
    analyzed_at: datetime

    @property
    def high_severity_count(self) -> int:
        """Count of high severity recommendations."""
        return sum(1 for r in self.recommendations if r.severity == 'high')

    @property
    def medium_severity_count(self) -> int:
        """Count of medium severity recommendations."""
        return sum(1 for r in self.recommendations if r.severity == 'medium')

    @property
    def low_severity_count(self) -> int:
        """Count of low severity recommendations."""
        return sum(1 for r in self.recommendations if r.severity == 'low')

    @property
    def total_recommendations(self) -> int:
        """Total count of recommendations."""
        return len(self.recommendations)

    def recommendations_by_category(self, category: str) -> List[Recommendation]:
        """Get recommendations filtered by category."""
        return [r for r in self.recommendations if r.category == category]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            'project_path': self.project_path,
            'project_name': self.project_name,
            'recommendations': [r.to_dict() for r in self.recommendations],
            'metrics': self.metrics,
            'analyzed_at': self.analyzed_at.isoformat() if self.analyzed_at else None,
            'summary': {
                'high_severity': self.high_severity_count,
                'medium_severity': self.medium_severity_count,
                'low_severity': self.low_severity_count,
                'total': self.total_recommendations,
            }
        }


__all__ = [
    'ConfigSource',
    'RepositoryConfig',
    'Skill',
    'MCP',
    'Command',
    'Plugin',
    'Hook',
    'Agent',
    'InheritanceLevel',
    'InheritanceChain',
    'ConfigurationFeatures',
    'Recommendation',
    'ProjectAnalysis',
]



"""Utilities for locating Claude Code data directories and files."""

from pathlib import Path
from typing import Optional


class ClaudeDataPaths:
    """Manages paths to Claude Code local data storage."""

    def __init__(self, base_dir: Optional[Path] = None):
        """
        Initialize Claude data paths.

        Args:
            base_dir: Optional custom base directory. Defaults to ~/.claude/
        """
        self.base_dir = base_dir or Path.home() / ".claude"

    def validate(self) -> bool:
        """Check if Claude data directory exists."""
        return self.base_dir.exists() and self.base_dir.is_dir()

    @property
    def history_file(self) -> Path:
        """Path to command history file (history.jsonl)."""
        return self.base_dir / "history.jsonl"

    @property
    def projects_dir(self) -> Path:
        """Path to projects directory containing session data."""
        return self.base_dir / "projects"

    @property
    def debug_dir(self) -> Path:
        """Path to debug logs directory."""
        return self.base_dir / "debug"

    @property
    def file_history_dir(self) -> Path:
        """Path to file history/backups directory."""
        return self.base_dir / "file-history"

    @property
    def todos_dir(self) -> Path:
        """Path to todos directory."""
        return self.base_dir / "todos"

    @property
    def settings_file(self) -> Path:
        """Path to settings.json file."""
        return self.base_dir / "settings.json"

    @property
    def shell_snapshots_dir(self) -> Path:
        """Path to shell snapshots directory."""
        return self.base_dir / "shell-snapshots"

    def get_project_session_files(self) -> list[Path]:
        """
        Get all session JSONL files from projects directory.

        Returns:
            List of paths to .jsonl session files
        """
        if not self.projects_dir.exists():
            return []

        session_files = []
        for project_dir in self.projects_dir.iterdir():
            if project_dir.is_dir():
                session_files.extend(project_dir.glob("*.jsonl"))

        return sorted(session_files, key=lambda p: p.stat().st_mtime, reverse=True)

    def get_debug_log_files(self) -> list[Path]:
        """
        Get all debug log files.

        Returns:
            List of paths to debug log files
        """
        if not self.debug_dir.exists():
            return []

        return sorted(
            self.debug_dir.glob("*.txt"),
            key=lambda p: p.stat().st_mtime,
            reverse=True
        )


def get_claude_paths() -> ClaudeDataPaths:
    """
    Get default Claude data paths instance.

    Returns:
        ClaudeDataPaths instance

    Raises:
        FileNotFoundError: If Claude data directory doesn't exist
    """
    paths = ClaudeDataPaths()
    if not paths.validate():
        raise FileNotFoundError(
            f"Claude data directory not found at {paths.base_dir}. "
            "Make sure Claude Code has been used at least once."
        )
    return paths



"""Token usage analyzer with cost calculations."""

from dataclasses import dataclass
from typing import Optional

from ..parsers.sessions import SessionParser, TokenUsage
from ..utils.time_filter import TimeFilter


# Friendly display names for models
MODEL_DISPLAY_NAMES = {
    'claude-opus-4-5-20251101': 'Claude Opus 4.5',
    'claude-opus-4-20250514': 'Claude Opus 4',
    'claude-sonnet-4-5-20250929': 'Claude Sonnet 4.5',
    'claude-sonnet-4-20250514': 'Claude Sonnet 4',
    'claude-haiku-4-5-20251001': 'Claude Haiku 4.5',
    # AWS Bedrock model IDs - Sonnet
    'eu.anthropic.claude-sonnet-4-5-20250929-v1:0': 'Claude Sonnet 4.5 (Bedrock EU)',
    'us.anthropic.claude-sonnet-4-5-20250929-v1:0': 'Claude Sonnet 4.5 (Bedrock US)',
    'anthropic.claude-sonnet-4-5-20250929-v1:0': 'Claude Sonnet 4.5 (Bedrock)',
    # AWS Bedrock model IDs - Opus
    'eu.anthropic.claude-opus-4-20250514-v1:0': 'Claude Opus 4 (Bedrock EU)',
    'us.anthropic.claude-opus-4-20250514-v1:0': 'Claude Opus 4 (Bedrock US)',
    'anthropic.claude-opus-4-20250514-v1:0': 'Claude Opus 4 (Bedrock)',
    'eu.anthropic.claude-opus-4-5-20251101-v1:0': 'Claude Opus 4.5 (Bedrock EU)',
    'us.anthropic.claude-opus-4-5-20251101-v1:0': 'Claude Opus 4.5 (Bedrock US)',
    'anthropic.claude-opus-4-5-20251101-v1:0': 'Claude Opus 4.5 (Bedrock)',
    # AWS Bedrock model IDs - Haiku
    'eu.anthropic.claude-haiku-4-5-20251001-v1:0': 'Claude Haiku 4.5 (Bedrock EU)',
    'us.anthropic.claude-haiku-4-5-20251001-v1:0': 'Claude Haiku 4.5 (Bedrock US)',
    'anthropic.claude-haiku-4-5-20251001-v1:0': 'Claude Haiku 4.5 (Bedrock)',
}


def get_model_display_name(model_id: str) -> str:
    """Get friendly display name for a model ID."""
    return MODEL_DISPLAY_NAMES.get(model_id, model_id)


# Claude API pricing by model (as of 2025)
# https://www.anthropic.com/pricing
MODEL_PRICING = {
    # Opus 4.5
    'claude-opus-4-5-20251101': {
        'input_per_mtok': 15.00,
        'output_per_mtok': 75.00,
        'cache_write_per_mtok': 18.75,
        'cache_read_per_mtok': 1.50,
    },
    # Sonnet 4.5
    'claude-sonnet-4-5-20250929': {
        'input_per_mtok': 3.00,
        'output_per_mtok': 15.00,
        'cache_write_per_mtok': 3.75,
        'cache_read_per_mtok': 0.30,
    },
    # Haiku 4.5
    'claude-haiku-4-5-20251001': {
        'input_per_mtok': 1.00,
        'output_per_mtok': 5.00,
        'cache_write_per_mtok': 1.25,
        'cache_read_per_mtok': 0.10,
    },
    # Opus 4
    'claude-opus-4-20250514': {
        'input_per_mtok': 15.00,
        'output_per_mtok': 75.00,
        'cache_write_per_mtok': 18.75,
        'cache_read_per_mtok': 1.50,
    },
}

# Bedrock pricing aliases - Sonnet
for region in ['eu', 'us', '']:
    prefix = f'{region}.anthropic' if region else 'anthropic'
    MODEL_PRICING[f'{prefix}.claude-sonnet-4-5-20250929-v1:0'] = MODEL_PRICING['claude-sonnet-4-5-20250929']

# Bedrock pricing aliases - Opus 4.5
for region in ['eu', 'us', '']:
    prefix = f'{region}.anthropic' if region else 'anthropic'
    MODEL_PRICING[f'{prefix}.claude-opus-4-5-20251101-v1:0'] = MODEL_PRICING['claude-opus-4-5-20251101']

# Bedrock pricing aliases - Opus 4
for region in ['eu', 'us', '']:
    prefix = f'{region}.anthropic' if region else 'anthropic'
    MODEL_PRICING[f'{prefix}.claude-opus-4-20250514-v1:0'] = MODEL_PRICING['claude-opus-4-20250514']

# Bedrock pricing aliases - Haiku 4.5
for region in ['eu', 'us', '']:
    prefix = f'{region}.anthropic' if region else 'anthropic'
    MODEL_PRICING[f'{prefix}.claude-haiku-4-5-20251001-v1:0'] = MODEL_PRICING['claude-haiku-4-5-20251001']

# Default pricing (Sonnet 4.5)
DEFAULT_PRICING = MODEL_PRICING['claude-sonnet-4-5-20250929']


@dataclass
class CostBreakdown:
    """Breakdown of costs by token type."""

    input_cost: float
    output_cost: float
    cache_write_cost: float
    cache_read_cost: float

    @property
    def total_cost(self) -> float:
        """Total cost across all token types."""
        return self.input_cost + self.output_cost + self.cache_write_cost + self.cache_read_cost

    @property
    def cache_savings(self) -> float:
        """
        Calculate savings from cache hits.
        Savings = what cache reads would have cost as regular input - actual cache read cost
        """
        # If cache reads were regular input tokens instead
        regular_cost = (self.cache_read_cost / DEFAULT_PRICING['cache_read_per_mtok']) * DEFAULT_PRICING['input_per_mtok']
        return regular_cost - self.cache_read_cost


@dataclass
class TokenSummary:
    """Summary of token usage and costs."""

    total_tokens: TokenUsage
    cost_breakdown: CostBreakdown
    cache_efficiency_pct: float

    @property
    def total_cost(self) -> float:
        """Total estimated cost."""
        return self.cost_breakdown.total_cost

    @property
    def net_cost(self) -> float:
        """Net cost after cache savings."""
        return self.cost_breakdown.total_cost

    @property
    def cache_savings(self) -> float:
        """Savings from cache efficiency."""
        return self.cost_breakdown.cache_savings

    def format_cost(self, cost: float) -> str:
        """Format cost as currency string."""
        return f"${cost:.2f}"


class TokenAnalyzer:
    """Analyzer for token usage and cost calculations."""

    def __init__(
        self,
        session_parser: SessionParser,
        time_filter: Optional[TimeFilter] = None
    ):
        """
        Initialize token analyzer.

        Args:
            session_parser: Parser for session data
            time_filter: Optional time filter
        """
        self.session_parser = session_parser
        self.time_filter = time_filter

    @staticmethod
    def calculate_cost(tokens: TokenUsage, model: Optional[str] = None) -> CostBreakdown:
        """
        Calculate costs for token usage based on model pricing.

        Args:
            tokens: TokenUsage instance
            model: Optional model name to get specific pricing

        Returns:
            CostBreakdown with calculated costs
        """
        # Get pricing for the specific model, or use default
        pricing = MODEL_PRICING.get(model, DEFAULT_PRICING)

        input_cost = (tokens.input_tokens / 1_000_000) * pricing['input_per_mtok']
        output_cost = (tokens.output_tokens / 1_000_000) * pricing['output_per_mtok']
        cache_write_cost = (tokens.cache_creation_input_tokens / 1_000_000) * pricing['cache_write_per_mtok']
        cache_read_cost = (tokens.cache_read_input_tokens / 1_000_000) * pricing['cache_read_per_mtok']

        return CostBreakdown(
            input_cost=input_cost,
            output_cost=output_cost,
            cache_write_cost=cache_write_cost,
            cache_read_cost=cache_read_cost
        )

    def get_summary(self) -> TokenSummary:
        """
        Get token usage summary with costs calculated per model.

        Returns:
            TokenSummary with aggregated data
        """
        # Get session stats
        stats = self.session_parser.get_stats(time_filter=self.time_filter)

        # Calculate costs per model and aggregate
        total_cost = CostBreakdown(
            input_cost=0.0,
            output_cost=0.0,
            cache_write_cost=0.0,
            cache_read_cost=0.0
        )

        for model, tokens in stats.model_usage.items():
            model_cost = self.calculate_cost(tokens, model)
            total_cost.input_cost += model_cost.input_cost
            total_cost.output_cost += model_cost.output_cost
            total_cost.cache_write_cost += model_cost.cache_write_cost
            total_cost.cache_read_cost += model_cost.cache_read_cost

        return TokenSummary(
            total_tokens=stats.total_tokens,
            cost_breakdown=total_cost,
            cache_efficiency_pct=stats.total_tokens.cache_efficiency_percentage
        )

    def get_model_breakdown(self) -> dict[str, tuple[TokenUsage, float]]:
        """
        Get per-model token usage and costs.

        Returns:
            Dict mapping model names to (TokenUsage, cost) tuples
        """
        stats = self.session_parser.get_stats(time_filter=self.time_filter)
        breakdown = {}

        for model, tokens in stats.model_usage.items():
            cost = self.calculate_cost(tokens, model)
            breakdown[model] = (tokens, cost.total_cost)

        # Sort by cost descending
        return dict(sorted(breakdown.items(), key=lambda x: x[1][1], reverse=True))

    def get_model_by_project_breakdown(self) -> dict[str, dict[str, tuple[TokenUsage, float]]]:
        """
        Get model usage breakdown per project.

        Returns:
            Dict mapping project names to dict of (model -> (TokenUsage, cost))
        """
        project_stats = self.session_parser.get_project_stats(time_filter=self.time_filter)
        result = {}

        for project, stats in project_stats.items():
            project_models = {}
            for model, tokens in stats.model_usage.items():
                cost = self.calculate_cost(tokens, model)
                project_models[model] = (tokens, cost.total_cost)

            if project_models:
                # Sort by cost descending
                result[project] = dict(sorted(project_models.items(), key=lambda x: x[1][1], reverse=True))

        # Sort projects by total cost descending
        return dict(sorted(result.items(),
                          key=lambda x: sum(c for _, c in x[1].values()),
                          reverse=True))

    def get_project_breakdown(self) -> dict[str, TokenSummary]:
        """
        Get per-project token usage and costs.

        Calculates costs per model within each project for accurate pricing.

        Returns:
            Dict mapping project paths to TokenSummary
        """
        project_stats = self.session_parser.get_project_stats(time_filter=self.time_filter)

        breakdown = {}
        for project, stats in project_stats.items():
            # Calculate costs per model and aggregate for accurate pricing
            total_cost = CostBreakdown(
                input_cost=0.0,
                output_cost=0.0,
                cache_write_cost=0.0,
                cache_read_cost=0.0
            )

            for model, tokens in stats.model_usage.items():
                model_cost = self.calculate_cost(tokens, model)
                total_cost.input_cost += model_cost.input_cost
                total_cost.output_cost += model_cost.output_cost
                total_cost.cache_write_cost += model_cost.cache_write_cost
                total_cost.cache_read_cost += model_cost.cache_read_cost

            breakdown[project] = TokenSummary(
                total_tokens=stats.total_tokens,
                cost_breakdown=total_cost,
                cache_efficiency_pct=stats.total_tokens.cache_efficiency_percentage
            )

        return breakdown

    @staticmethod
    def format_token_count(count: int) -> str:
        """
        Format token count in human-readable form.

        Args:
            count: Token count

        Returns:
            Formatted string (e.g., "1.2M", "458K", "1234")
        """
        if count >= 1_000_000:
            return f"{count / 1_000_000:.1f}M"
        elif count >= 1_000:
            return f"{count / 1_000:.0f}K"
        else:
            return str(count)



"""Dashboard route handler for Claude Monitor web application."""

from datetime import datetime, timedelta
from typing import Any, Dict
from flask import Blueprint, render_template, current_app
import logging

logger = logging.getLogger(__name__)

dashboard_bp = Blueprint('dashboard', __name__)


@dashboard_bp.route('/')
@dashboard_bp.route('/dashboard')
def index() -> str:
    """
    Render the main dashboard page with usage statistics and project breakdown.

    Fetches data from the DashboardService and passes it to the dashboard template.

    Returns:
        Rendered HTML template with dashboard data or error page

    Raises:
        RuntimeError: If dashboard_service is not available on the app
    """
    try:
        # Get the dashboard service from the app context
        if not hasattr(current_app, 'dashboard_service'):
            logger.error('Dashboard service not found on current app')
            return render_template(
                'pages/error.html',
                error_title='Configuration Error',
                error_message='Dashboard service not configured. Please restart the server.',
                error_details='The dashboard service was not properly initialized.'
            ), 500

        service = current_app.dashboard_service

        # Fetch all dashboard data with error handling
        usage_summary: Dict[str, Any] = service.get_usage_summary()
        token_summary: Dict[str, Any] = service.get_token_summary()
        project_breakdown: Dict[str, Any] = service.get_project_breakdown()
        model_breakdown: Dict[str, Any] = service.get_model_breakdown()
        token_trend: Dict[str, Any] = service.get_daily_token_trend(days=14)
        cost_trend: Dict[str, Any] = service.get_daily_cost_trend(days=14)
        project_cost_trend: Dict[str, Any] = service.get_project_cost_trend(days=14, max_projects=8)

        logger.debug('Dashboard data fetched successfully')

        # Render template with all data
        return render_template(
            'pages/dashboard.html',
            usage=usage_summary,
            tokens=token_summary,
            projects=project_breakdown,
            models=model_breakdown,
            token_trend=token_trend,
            cost_trend=cost_trend,
            project_cost_trend=project_cost_trend
        )

    except ValueError as e:
        # Handle data parsing errors (like invalid timestamps)
        logger.error(f'Data parsing error: {e}', exc_info=True)
        return render_template(
            'pages/error.html',
            error_title='Data Parsing Error',
            error_message='Unable to parse Claude Code usage data.',
            error_details=f'There appears to be corrupted or invalid data in your ~/.claude/ directory. Error: {str(e)}',
            suggestion='Try using the CLI tool to see if it shows more details: claude-monitor'
        ), 500

    except FileNotFoundError as e:
        # Handle missing data directory
        logger.error(f'Claude data not found: {e}')
        return render_template(
            'pages/error.html',
            error_title='No Claude Data Found',
            error_message='Unable to find Claude Code usage data.',
            error_details='Make sure you have used Claude Code at least once before running the monitor.',
            suggestion='Run Claude Code first, then come back here to view your usage statistics.'
        ), 404

    except Exception as e:
        # Handle any other errors
        logger.error(f'Unexpected error loading dashboard: {e}', exc_info=True)
        return render_template(
            'pages/error.html',
            error_title='Unexpected Error',
            error_message='An unexpected error occurred while loading the dashboard.',
            error_details=str(e),
            suggestion='Please check the server logs for more details or report this issue.'
        ), 500


@dashboard_bp.route('/tokens')
def tokens() -> str:
    """
    Render the tokens page with detailed token usage breakdown.

    Returns:
        Rendered HTML template with token data or error page
    """
    try:
        service = current_app.dashboard_service
        token_summary: Dict[str, Any] = service.get_token_summary()
        model_breakdown: Dict[str, Any] = service.get_model_breakdown()
        available_models = service.get_available_models()

        return render_template(
            'pages/tokens.html',
            tokens=token_summary,
            models=model_breakdown,
            available_models=available_models
        )

    except ValueError as e:
        logger.error(f'Data parsing error: {e}', exc_info=True)
        return render_template(
            'pages/error.html',
            error_title='Data Parsing Error',
            error_message='Unable to parse Claude Code usage data.',
            error_details=f'Error: {str(e)}',
            suggestion='Try using the CLI tool: claude-monitor'
        ), 500

    except Exception as e:
        logger.error(f'Unexpected error loading tokens page: {e}', exc_info=True)
        return render_template(
            'pages/error.html',
            error_title='Unexpected Error',
            error_message='An unexpected error occurred.',
            error_details=str(e)
        ), 500


@dashboard_bp.route('/projects')
def projects() -> str:
    """
    Render the projects page with per-project analytics.

    Returns:
        Rendered HTML template with project data or error page
    """
    try:
        service = current_app.dashboard_service
        project_breakdown: Dict[str, Any] = service.get_project_breakdown()
        available_models = service.get_available_models()


        return render_template(
            'pages/projects.html',
            projects=project_breakdown,
            available_models=available_models
        )

    except ValueError as e:
        logger.error(f'Data parsing error: {e}', exc_info=True)
        return render_template(
            'pages/error.html',
            error_title='Data Parsing Error',
            error_message='Unable to parse Claude Code usage data.',
            error_details=f'Error: {str(e)}',
            suggestion='Try using the CLI tool: claude-monitor'
        ), 500

    except Exception as e:
        logger.error(f'Unexpected error loading projects page: {e}', exc_info=True)
        return render_template(
            'pages/error.html',
            error_title='Unexpected Error',
            error_message='An unexpected error occurred.',
            error_details=str(e)
        ), 500


@dashboard_bp.route('/files')
def files() -> str:
    """Render the files page with file operation statistics."""
    try:
        service = current_app.dashboard_service

        # Get file statistics
        file_data = service.get_file_statistics(limit=20)

        return render_template('pages/files.html', files=file_data)

    except Exception as e:
        logger.error(f'Error loading files page: {e}', exc_info=True)
        return render_template('pages/files.html', files={})


@dashboard_bp.route('/integrations')
def integrations() -> str:
    """Render the integrations page with MCP server statistics."""
    try:
        service = current_app.dashboard_service

        # Get MCP integration data
        mcp_data = service.get_mcp_integrations()
        available_models = service.get_available_models()

        # Format data for template
        integrations_data = {
            'active_count': mcp_data['total_servers'],
            'most_used_ide': 'CLI',  # Claude Code is CLI-based
            'mcp_servers': mcp_data['total_servers'],
            'total_calls': mcp_data['total_calls'],
            'total_tokens': mcp_data.get('total_tokens', 0),
            'total_cost': mcp_data.get('total_cost', 0.0),
            'most_used_server': mcp_data['most_used_server'],
            'server_activity': mcp_data['servers']
        }

        return render_template(
            'pages/integrations.html',
            integrations=integrations_data,
            available_models=available_models
        )

    except Exception as e:
        logger.error(f'Error loading integrations page: {e}', exc_info=True)
        return render_template('pages/integrations.html', integrations={
            'most_used_ide': 'CLI',
            'server_activity': []
        })


@dashboard_bp.route('/features')
def features() -> str:
    """Render the features page with tool usage statistics."""
    try:
        service = current_app.dashboard_service

        # Get tool usage data
        top_tools = service.get_top_tools(limit=20)
        features_summary = service.get_features_summary()

        # Build tool_usage list with proper field names for template
        tool_usage = []
        for tool in top_tools.get('tools', []):
            tool_usage.append({
                'name': tool.get('tool_name', ''),
                'category': tool.get('category', 'tool'),
                'usage_count': tool.get('invocation_count', 0),
                'total_tokens': tool.get('total_tokens', 0),
                'cost': tool.get('total_cost', 0.0),
            })

        features_data = {
            'total_tools': len(tool_usage),
            'total_calls': top_tools.get('total_calls', 0),
            'total_tokens': top_tools.get('total_tokens', 0),
            'total_cost': top_tools.get('total_cost', 0.0),
            'sub_agents_used': features_summary.get('subagents', {}).get('unique_used', 0),
            'tool_usage': tool_usage,
            'categories': features_summary.get('categories', {}),
        }

        return render_template('pages/features.html', features=features_data)

    except Exception as e:
        logger.error(f'Error loading features page: {e}', exc_info=True)
        return render_template('pages/features.html', features={})


@dashboard_bp.route('/configuration')
def configuration() -> str:
    """Render the configuration page with Claude Code configuration viewer."""
    from flask import request

    try:
        service = current_app.dashboard_service

        # Get discovered repositories
        repositories = service.get_discovered_repositories()

        # Get selected repo from query params, default to first (User Configuration)
        selected_repo_path = request.args.get('repo', repositories[0]['path'] if repositories else None)

        # Find the selected repository
        selected_repo = None
        if selected_repo_path:
            for repo in repositories:
                if repo['path'] == selected_repo_path:
                    selected_repo = repo
                    break

        # Default to first repo if not found
        if not selected_repo and repositories:
            selected_repo = repositories[0]
            selected_repo_path = repositories[0]['path']

        # Get features for selected repository
        features = {}
        if selected_repo_path:
            features = service.get_configuration_features(selected_repo_path)

        return render_template(
            'pages/configuration.html',
            repositories=repositories,
            selected_repo=selected_repo,
            features=features
        )

    except Exception as e:
        logger.error(f'Error loading configuration page: {e}', exc_info=True)
        return render_template(
            'pages/configuration.html',
            repositories=[],
            selected_repo=None,
            features={}
        )


@dashboard_bp.route('/api/configuration/<path:repo_path>')
def api_configuration(repo_path: str) -> str:
    """
    API endpoint for HTMX to fetch configuration features for a specific repository.

    Args:
        repo_path: Path to the repository

    Returns:
        Rendered HTML partial with feature list
    """
    try:
        service = current_app.dashboard_service
        # Flask's path converter strips leading slash, so add it back if missing
        if not repo_path.startswith('/'):
            repo_path = '/' + repo_path
        features = service.get_configuration_features(repo_path)

        return render_template(
            'partials/feature_list.html',
            features=features
        )

    except Exception as e:
        # Sanitize user-provided repo_path to prevent log injection
        safe_repo_path = repo_path.replace('\r', '').replace('\n', '')
        # Sanitize exception message to prevent log injection via newline characters
        safe_error = str(e).replace('\r', '').replace('\n', '')
        logger.error(f'Error loading configuration for repo {safe_repo_path}: {safe_error}', exc_info=True)
        return render_template(
            'partials/feature_list.html',
            features={}
        )


@dashboard_bp.route('/api/feature-detail/<path:repo_path>/<feature_type>/<feature_id>')
def api_feature_detail(repo_path: str, feature_type: str, feature_id: str) -> str:
    """
    API endpoint for HTMX to fetch detailed information about a specific feature.

    Args:
        repo_path: Path to the repository
        feature_type: Type of feature (skill, mcp, command, plugin, hook, agent)
        feature_id: Name/ID of the feature

    Returns:
        Rendered HTML partial with feature details
    """
    try:
        service = current_app.dashboard_service
        # Flask's path converter strips leading slash, so add it back if missing
        if not repo_path.startswith('/'):
            repo_path = '/' + repo_path
        detail = service.get_feature_detail(repo_path, feature_type, feature_id)

        # Render appropriate detail template based on feature type
        template_map = {
            'skill': 'partials/details/skill_detail.html',
            'mcp': 'partials/details/mcp_detail.html',
            'command': 'partials/details/command_detail.html',
            'plugin': 'partials/details/plugin_detail.html',
            'hook': 'partials/details/hook_detail.html',
            'agent': 'partials/details/agent_detail.html',
        }

        template = template_map.get(feature_type, 'partials/details/feature_detail.html')

        return render_template(
            template,
            feature=detail,
            feature_type=feature_type
        )

    except Exception as e:
        safe_feature_type = feature_type.replace('\r', '').replace('\n', '')
        safe_feature_id = feature_id.replace('\r', '').replace('\n', '')
        logger.error(f'Error loading detail for {safe_feature_type} {safe_feature_id}: {e}', exc_info=True)
        return f'<div class="alert alert-error">Error loading feature details: {str(e)}</div>'


@dashboard_bp.route('/export/configuration/<path:repo_path>')
def export_configuration(repo_path: str):
    """
    Export configuration features as JSON.

    Args:
        repo_path: Path to the repository

    Returns:
        JSON file download
    """
    from flask import jsonify, Response
    import json as json_lib

    try:
        service = current_app.dashboard_service
        # Flask's path converter strips leading slash, so add it back if missing
        if not repo_path.startswith('/'):
            repo_path = '/' + repo_path
        features = service.get_configuration_features(repo_path)

        # Find repository name
        repos = service.get_discovered_repositories()
        repo_name = "configuration"
        for repo in repos:
            if repo['path'] == repo_path:
                repo_name = repo['name'].replace(' ', '_').lower()
                break

        # Create JSON response
        json_data = json_lib.dumps(features, indent=2, default=str)

        return Response(
            json_data,
            mimetype='application/json',
            headers={
                'Content-Disposition': f'attachment; filename={repo_name}_configuration.json'
            }
        )
    except Exception as e:
        logger.error(f'Error exporting configuration: {e}', exc_info=True)
        return jsonify({'error': str(e)}), 500


@dashboard_bp.route('/api/dashboard')
def api_dashboard() -> str:
    """
    API endpoint for HTMX to fetch filtered dashboard data.
    
    Query params:
        period: today|week|month|all (default: all)
    
    Returns:
        Rendered HTML partial with dashboard content
    """
    from flask import request
    from ...utils.time_filter import TimeFilter
    
    try:
        service = current_app.dashboard_service
        period = request.args.get('period', 'all')
        
        # Create time filter based on period
        time_filter = None
        if period != 'all':
            now = datetime.now()
            if period == 'today':
                start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
            elif period == 'week':
                start_time = now - timedelta(days=7)
            elif period == 'month':
                start_time = now - timedelta(days=30)
            else:
                start_time = None
            
            if start_time:
                time_filter = TimeFilter(start_time=start_time, end_time=now)
        
        # Fetch filtered data
        usage_summary: Dict[str, Any] = service.get_usage_summary(time_filter=time_filter)
        token_summary: Dict[str, Any] = service.get_token_summary(time_filter=time_filter)
        project_breakdown: Dict[str, Any] = service.get_project_breakdown(time_filter=time_filter)
        model_breakdown: Dict[str, Any] = service.get_model_breakdown(time_filter=time_filter)

        # For "All Time", show last 14 days on chart for better context
        chart_days = 14 if period == 'all' else 7
        token_trend: Dict[str, Any] = service.get_daily_token_trend(days=chart_days, time_filter=time_filter)
        cost_trend: Dict[str, Any] = service.get_daily_cost_trend(days=chart_days, time_filter=time_filter)
        project_cost_trend: Dict[str, Any] = service.get_project_cost_trend(days=chart_days, time_filter=time_filter, max_projects=8)

        # Render partial template
        return render_template(
            'partials/dashboard_content.html',
            usage=usage_summary,
            tokens=token_summary,
            projects=project_breakdown,
            models=model_breakdown,
            token_trend=token_trend,
            cost_trend=cost_trend,
            project_cost_trend=project_cost_trend
        )
    
    except Exception as e:
        logger.error(f'Error loading filtered dashboard: {e}', exc_info=True)
        return f'<div class="text-red-600 p-4">Error loading data: {str(e)}</div>', 500


@dashboard_bp.route('/api/projects')
def api_projects() -> str:
    """
    API endpoint for HTMX to fetch filtered projects data.

    Query params:
        period: today|week|month|all (default: all)
        model: model ID to filter by (default: all models)

    Returns:
        Rendered HTML partial with projects content
    """
    from flask import request
    from ...utils.time_filter import TimeFilter

    try:
        service = current_app.dashboard_service
        period = request.args.get('period', 'all')
        model = request.args.get('model', '')

        # Create time filter based on period
        time_filter = None
        if period != 'all':
            now = datetime.now()
            if period == 'today':
                start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
            elif period == 'week':
                start_time = now - timedelta(days=7)
            elif period == 'month':
                start_time = now - timedelta(days=30)
            else:
                start_time = None

            if start_time:
                time_filter = TimeFilter(start_time=start_time, end_time=now)

        # Fetch data - filtered by model if specified
        if model:
            project_breakdown = service.get_project_breakdown_by_model(model, time_filter=time_filter)
        else:
            project_breakdown = service.get_project_breakdown(time_filter=time_filter)

        available_models = service.get_available_models(time_filter=time_filter)

        # Render partial content
        return render_template(
            'partials/projects_content.html',
            projects=project_breakdown,
            available_models=available_models,
            selected_model=model
        )

    except Exception as e:
        logger.error(f'Error loading filtered projects: {e}', exc_info=True)
        return f'<div class="text-red-600 p-4">Error loading projects: {str(e)}</div>', 500


@dashboard_bp.route('/api/files')
def api_files() -> str:
    """API endpoint for HTMX to fetch filtered files data."""
    from flask import request
    from ...utils.time_filter import TimeFilter

    try:
        service = current_app.dashboard_service
        period = request.args.get('period', 'all')

        time_filter = None
        if period != 'all':
            now = datetime.now()
            if period == 'today':
                start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
            elif period == 'week':
                start_time = now - timedelta(days=7)
            elif period == 'month':
                start_time = now - timedelta(days=30)
            else:
                start_time = None

            if start_time:
                time_filter = TimeFilter(start_time=start_time, end_time=now)

        file_data = service.get_file_statistics(limit=20, time_filter=time_filter)
        return render_template('partials/files_content.html', files=file_data)

    except Exception as e:
        logger.error(f'Error loading filtered files: {e}', exc_info=True)
        return f'<div class="text-red-600 p-4">Error loading files: {str(e)}</div>', 500


@dashboard_bp.route('/api/integrations')
def api_integrations() -> str:
    """API endpoint for HTMX to fetch filtered integrations data."""
    from flask import request
    from ...utils.time_filter import TimeFilter

    try:
        service = current_app.dashboard_service
        period = request.args.get('period', 'all')

        time_filter = None
        if period != 'all':
            now = datetime.now()
            if period == 'today':
                start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
            elif period == 'week':
                start_time = now - timedelta(days=7)
            elif period == 'month':
                start_time = now - timedelta(days=30)
            else:
                start_time = None

            if start_time:
                time_filter = TimeFilter(start_time=start_time, end_time=now)

        mcp_data = service.get_mcp_integrations(time_filter=time_filter)
        integrations_data = {
            'active_count': mcp_data['total_servers'],
            'total_calls': mcp_data['total_calls'],
            'total_tokens': mcp_data.get('total_tokens', 0),
            'total_cost': mcp_data.get('total_cost', 0.0),
            'server_activity': mcp_data['servers']
        }
        return render_template('partials/integrations_content.html', integrations=integrations_data)

    except Exception as e:
        logger.error(f'Error loading filtered integrations: {e}', exc_info=True)
        return f'<div class="text-red-600 p-4">Error loading integrations: {str(e)}</div>', 500


@dashboard_bp.route('/api/tokens')
def api_tokens() -> str:
    """API endpoint for HTMX to fetch filtered tokens data."""
    from flask import request
    from ...utils.time_filter import TimeFilter

    try:
        service = current_app.dashboard_service
        period = request.args.get('period', 'all')
        model = request.args.get('model', '')

        time_filter = None
        if period != 'all':
            now = datetime.now()
            if period == 'today':
                start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
            elif period == 'week':
                start_time = now - timedelta(days=7)
            elif period == 'month':
                start_time = now - timedelta(days=30)
            else:
                start_time = None

            if start_time:
                time_filter = TimeFilter(start_time=start_time, end_time=now)

        # Get token data - filtered by model if specified
        if model:
            token_summary = service.get_token_summary_by_model(model, time_filter=time_filter)
        else:
            token_summary = service.get_token_summary(time_filter=time_filter)

        model_breakdown = service.get_model_breakdown(time_filter=time_filter)
        available_models = service.get_available_models(time_filter=time_filter)

        return render_template(
            'partials/tokens_content.html',
            tokens=token_summary,
            models=model_breakdown,
            available_models=available_models,
            selected_model=model
        )

    except Exception as e:
        logger.error(f'Error loading filtered tokens: {e}', exc_info=True)
        return f'<div class="text-red-600 p-4">Error loading tokens: {str(e)}</div>', 500


@dashboard_bp.route('/api/features')
def api_features() -> str:
    """API endpoint for HTMX to fetch filtered features data."""
    from flask import request
    from ...utils.time_filter import TimeFilter

    try:
        service = current_app.dashboard_service
        period = request.args.get('period', 'all')

        time_filter = None
        if period != 'all':
            now = datetime.now()
            if period == 'today':
                start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
            elif period == 'week':
                start_time = now - timedelta(days=7)
            elif period == 'month':
                start_time = now - timedelta(days=30)
            else:
                start_time = None

            if start_time:
                time_filter = TimeFilter(start_time=start_time, end_time=now)

        # Get tool usage data
        top_tools = service.get_top_tools(limit=20, time_filter=time_filter)
        features_summary = service.get_features_summary(time_filter=time_filter)

        # Build tool_usage list with proper field names for template
        tool_usage = []
        for tool in top_tools.get('tools', []):
            tool_usage.append({
                'name': tool.get('tool_name', ''),
                'category': tool.get('category', 'tool'),
                'usage_count': tool.get('invocation_count', 0),
                'total_tokens': tool.get('total_tokens', 0),
                'cost': tool.get('total_cost', 0.0),
            })

        features_data = {
            'total_tools': len(tool_usage),
            'total_calls': top_tools.get('total_calls', 0),
            'total_tokens': top_tools.get('total_tokens', 0),
            'total_cost': top_tools.get('total_cost', 0.0),
            'sub_agents_used': features_summary.get('subagents', {}).get('unique_used', 0),
            'tool_usage': tool_usage,
            'categories': features_summary.get('categories', {}),
        }

        return render_template('partials/features_content.html', features=features_data)

    except Exception as e:
        logger.error(f'Error loading filtered features: {e}', exc_info=True)
        return f'<div class="text-red-600 p-4">Error loading features: {str(e)}</div>', 500


@dashboard_bp.route('/api/project/analyze')
def api_project_analyze() -> Any:
    """
    API endpoint to analyze a project for optimization recommendations.

    Query params:
        project_path: Full path to the project
        project_name: Display name of the project
        period: today|week|month|all (default: all)

    Returns:
        JSON with project analysis and recommendations
    """
    from flask import request, jsonify
    from ...utils.time_filter import TimeFilter
    from urllib.parse import unquote

    try:
        service = current_app.dashboard_service
        project_path = request.args.get('project_path', '')
        project_name = request.args.get('project_name', 'Unknown')
        period = request.args.get('period', 'all')

        # URL decode the project path
        if project_path:
            project_path = unquote(project_path)

        # Create time filter based on period
        time_filter = None
        if period != 'all':
            now = datetime.now()
            if period == 'today':
                start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
            elif period == 'week':
                start_time = now - timedelta(days=7)
            elif period == 'month':
                start_time = now - timedelta(days=30)
            else:
                start_time = None

            if start_time:
                time_filter = TimeFilter(start_time=start_time, end_time=now)

        # Get project analysis
        analysis = service.get_project_analysis(
            project_path=project_path,
            project_name=project_name,
            time_filter=time_filter
        )

        return jsonify(analysis), 200

    except Exception as e:
        logger.error(f'Error analyzing project: {e}')
        return jsonify({
            'error': 'An error occurred while analyzing the project',
            'project_name': project_name,
            'recommendations': [],
            'metrics': {},
            'summary': {'total': 0, 'high_severity': 0, 'medium_severity': 0, 'low_severity': 0}
        }), 500


@dashboard_bp.route('/export/<format>')
def export_data(format: str) -> Any:
    """
    Export dashboard data in various formats.
    
    Args:
        format: csv or json
    
    Returns:
        Downloaded file
    """
    import csv
    import io
    from flask import make_response
    
    try:
        service = current_app.dashboard_service
        
        if format not in ['csv', 'json']:
            return "Invalid format. Use 'csv' or 'json'", 400
        
        # Fetch all data
        usage_summary = service.get_usage_summary()
        token_summary = service.get_token_summary()
        project_breakdown = service.get_project_breakdown()
        
        if format == 'json':
            # Export as JSON
            from flask import jsonify
            data = {
                'usage': usage_summary,
                'tokens': token_summary,
                'projects': project_breakdown,
                'exported_at': datetime.now().isoformat()
            }
            response = make_response(jsonify(data))
            response.headers['Content-Disposition'] = 'attachment; filename=claude_monitor_export.json'
            response.headers['Content-Type'] = 'application/json'
            return response
        
        elif format == 'csv':
            # Export as CSV
            output = io.StringIO()
            writer = csv.writer(output)
            
            # Write usage summary
            writer.writerow(['Usage Summary'])
            writer.writerow(['Metric', 'Value'])
            writer.writerow(['Total Commands', usage_summary.get('total_commands', 0)])
            writer.writerow(['Total Sessions', usage_summary.get('total_sessions', 0)])
            writer.writerow(['Total Messages', usage_summary.get('total_messages', 0)])
            writer.writerow(['Active Projects', usage_summary.get('total_projects', 0)])
            writer.writerow([])
            
            # Write token summary
            writer.writerow(['Token Summary'])
            writer.writerow(['Token Type', 'Count', 'Cost'])
            writer.writerow(['Input Tokens', token_summary.get('input_tokens', 0), f"${token_summary.get('input_cost', 0):.2f}"])
            writer.writerow(['Output Tokens', token_summary.get('output_tokens', 0), f"${token_summary.get('output_cost', 0):.2f}"])
            writer.writerow(['Cache Creation', token_summary.get('cache_creation_tokens', 0), f"${token_summary.get('cache_creation_cost', 0):.2f}"])
            writer.writerow(['Cache Reads', token_summary.get('cache_read_tokens', 0), f"${token_summary.get('cache_read_cost', 0):.2f}"])
            writer.writerow(['Total Cost', '-', f"${token_summary.get('total_cost', 0):.2f}"])
            writer.writerow([])
            
            # Write projects
            if project_breakdown.get('projects'):
                writer.writerow(['Projects'])
                writer.writerow(['Name', 'Commands', 'Sessions', 'Messages', 'Total Tokens', 'Cost'])
                for project in project_breakdown['projects']:
                    writer.writerow([
                        project.get('name', ''),
                        project.get('command_count', 0),
                        project.get('session_count', 0),
                        project.get('message_count', 0),
                        project.get('total_tokens', 0),
                        f"${project.get('total_cost', 0):.2f}"
                    ])
            
            response = make_response(output.getvalue())
            response.headers['Content-Disposition'] = 'attachment; filename=claude_monitor_export.csv'
            response.headers['Content-Type'] = 'text/csv'
            return response
    
    except Exception as e:
        logger.error(f'Error exporting data: {e}', exc_info=True)
        return 'Error exporting data. Please check the server logs for more details.', 500



"""Analyzer for Claude Code features (sub-agents, skills, MCPs)."""

from dataclasses import dataclass
from typing import Optional

from ..parsers.tools import ToolUsageParser, ToolStats
from ..parsers.skills import SkillsParser, SkillInfo, ConfigurationParser
from ..parsers.debug import DebugLogParser, MCPServerStats
from ..utils.time_filter import TimeFilter


@dataclass
class FeaturesSummary:
    """Summary of Claude Code features usage."""

    # Sub-agents
    subagent_stats: dict[str, ToolStats]
    total_subagent_calls: int
    unique_subagents_used: int

    # Skills
    installed_skills: list[SkillInfo]
    total_skills: int

    # MCPs
    mcp_stats: dict[str, MCPServerStats]
    enabled_mcps: dict[str, bool]
    total_mcp_servers: int

    # Tools (general)
    tool_stats: dict[str, ToolStats]
    total_tool_calls: int

    # Configuration
    always_thinking_enabled: bool
    enabled_plugins: dict[str, bool]


class FeaturesAnalyzer:
    """Analyzer for Claude Code features."""

    def __init__(
        self,
        tool_parser: ToolUsageParser,
        skills_parser: SkillsParser,
        debug_parser: DebugLogParser,
        config_parser: ConfigurationParser,
        time_filter: Optional[TimeFilter] = None
    ):
        """
        Initialize features analyzer.

        Args:
            tool_parser: Parser for tool usage
            skills_parser: Parser for skills
            debug_parser: Parser for debug logs (MCP data)
            config_parser: Parser for configuration
            time_filter: Optional time filter
        """
        self.tool_parser = tool_parser
        self.skills_parser = skills_parser
        self.debug_parser = debug_parser
        self.config_parser = config_parser
        self.time_filter = time_filter

    def get_summary(self) -> FeaturesSummary:
        """
        Get comprehensive features summary.

        Returns:
            FeaturesSummary with all feature data
        """
        # Get sub-agent stats
        subagent_stats = self.tool_parser.get_subagent_stats(time_filter=self.time_filter)
        total_subagent_calls = sum(s.invocation_count for s in subagent_stats.values())
        unique_subagents = len(subagent_stats)

        # Get tool stats
        tool_stats = self.tool_parser.get_tool_stats(time_filter=self.time_filter)
        total_tool_calls = sum(s.invocation_count for s in tool_stats.values())

        # Get skills
        installed_skills = self.skills_parser.get_installed_skills()
        total_skills = len(installed_skills)

        # Get MCP stats
        mcp_stats = self.debug_parser.get_all_mcp_stats()
        total_mcp_servers = len(mcp_stats)

        # Get configuration
        enabled_plugins = self.config_parser.get_enabled_plugins()
        always_thinking = self.config_parser.is_always_thinking_enabled()

        # Try to correlate enabled plugins with MCP servers
        # Plugins might enable MCP servers
        enabled_mcps = {}
        for plugin_name, enabled in enabled_plugins.items():
            if 'mcp' in plugin_name.lower():
                enabled_mcps[plugin_name] = enabled

        return FeaturesSummary(
            subagent_stats=subagent_stats,
            total_subagent_calls=total_subagent_calls,
            unique_subagents_used=unique_subagents,
            installed_skills=installed_skills,
            total_skills=total_skills,
            mcp_stats=mcp_stats,
            enabled_mcps=enabled_mcps,
            total_mcp_servers=total_mcp_servers,
            tool_stats=tool_stats,
            total_tool_calls=total_tool_calls,
            always_thinking_enabled=always_thinking,
            enabled_plugins=enabled_plugins
        )

    def get_top_tools(self, limit: int = 10) -> list[tuple[str, ToolStats]]:
        """
        Get most frequently used tools.

        Args:
            limit: Maximum number of tools to return

        Returns:
            List of (tool_name, ToolStats) tuples
        """
        tool_stats = self.tool_parser.get_tool_stats(time_filter=self.time_filter)
        return list(tool_stats.items())[:limit]

    def get_top_subagents(self, limit: int = 10) -> list[tuple[str, ToolStats]]:
        """
        Get most frequently used sub-agents.

        Args:
            limit: Maximum number of sub-agents to return

        Returns:
            List of (subagent_type, ToolStats) tuples
        """
        subagent_stats = self.tool_parser.get_subagent_stats(time_filter=self.time_filter)
        return list(subagent_stats.items())[:limit]



"""Parser for Claude Code debug logs to extract MCP server usage."""

import re
from collections import defaultdict
from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional


@dataclass
class MCPServerStats:
    """Statistics for MCP server usage."""

    server_name: str
    connection_count: int = 0
    tool_calls: list[str] = field(default_factory=list)
    errors: list[str] = field(default_factory=list)
    projects: set[str] = field(default_factory=set)  # Projects using this MCP

    @property
    def tool_call_count(self) -> int:
        """Total number of tool calls."""
        return len(self.tool_calls)

    @property
    def error_count(self) -> int:
        """Total number of errors."""
        return len(self.errors)

    @property
    def unique_tools(self) -> set[str]:
        """Set of unique tool names called."""
        return set(self.tool_calls)


class DebugLogParser:
    """Parser for Claude Code debug logs."""

    # Regex patterns for extracting information
    MCP_CONNECTION_PATTERN = re.compile(r'MCP server "([^"]+)":\s*(Initializing|Starting|connected|started)', re.IGNORECASE)
    MCP_TOOL_PATTERN = re.compile(r'mcp__([^_]+)__([^",\s\]]+)')
    ERROR_PATTERN = re.compile(r'(ERROR|Error|error):\s*(.+)')

    def __init__(self, debug_files: list[Path], project_map: Optional[dict[str, str]] = None):
        """
        Initialize debug log parser.

        Args:
            debug_files: List of debug log file paths
            project_map: Optional dict mapping session IDs to project paths
        """
        self.debug_files = debug_files
        self.project_map = project_map or {}

    def parse_file(self, debug_file: Path) -> dict[str, MCPServerStats]:
        """
        Parse a single debug log file.

        Args:
            debug_file: Path to debug log file

        Returns:
            Dict mapping server names to MCPServerStats
        """
        stats: dict[str, MCPServerStats] = defaultdict(
            lambda: MCPServerStats(server_name="")
        )

        if not debug_file.exists():
            return {}

        # Get project path from session ID
        session_id = debug_file.stem  # filename without extension
        project_path = self.project_map.get(session_id, "global")

        try:
            with open(debug_file, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()

            # Find MCP server connections
            for match in self.MCP_CONNECTION_PATTERN.finditer(content):
                server_name = match.group(1)
                if server_name not in stats:
                    stats[server_name] = MCPServerStats(server_name=server_name)
                stats[server_name].connection_count += 1
                stats[server_name].projects.add(project_path)

            # Find MCP tool calls (matches mcp__servername__toolname)
            for match in self.MCP_TOOL_PATTERN.finditer(content):
                server_name = match.group(1)
                tool_name = match.group(2)
                full_tool_name = f"mcp__{server_name}__{tool_name}"

                if server_name not in stats:
                    stats[server_name] = MCPServerStats(server_name=server_name)
                stats[server_name].tool_calls.append(full_tool_name)
                stats[server_name].projects.add(project_path)

            # Find errors (basic error detection)
            for match in self.ERROR_PATTERN.finditer(content):
                error_msg = match.group(2).strip()
                # Try to associate with a server if mentioned
                for server_name in stats.keys():
                    if server_name in error_msg:
                        stats[server_name].errors.append(error_msg)
                        break

        except Exception:
            # Skip files that can't be read
            pass

        return dict(stats)

    def get_all_mcp_stats(self) -> dict[str, MCPServerStats]:
        """
        Get aggregated MCP server statistics from all debug logs.

        Returns:
            Dict mapping server names to aggregated MCPServerStats
        """
        aggregated: dict[str, MCPServerStats] = {}

        for debug_file in self.debug_files:
            file_stats = self.parse_file(debug_file)

            for server_name, stats in file_stats.items():
                if server_name not in aggregated:
                    aggregated[server_name] = MCPServerStats(server_name=server_name)

                aggregated[server_name].connection_count += stats.connection_count
                aggregated[server_name].tool_calls.extend(stats.tool_calls)
                aggregated[server_name].errors.extend(stats.errors)
                aggregated[server_name].projects.update(stats.projects)

        # Sort by tool call count (descending)
        return dict(
            sorted(
                aggregated.items(),
                key=lambda x: x[1].tool_call_count,
                reverse=True
            )
        )

    def get_total_error_count(self) -> int:
        """
        Get total number of errors across all logs.

        Returns:
            Total error count
        """
        stats = self.get_all_mcp_stats()
        return sum(s.error_count for s in stats.values())



"""Parser for Claude Code session files containing token usage data."""

import json
from dataclasses import dataclass, field
from datetime import datetime
from pathlib import Path
from typing import Iterator, Optional

from ..utils.time_filter import TimeFilter


@dataclass
class TokenUsage:
    """Represents token usage for a single message."""

    input_tokens: int = 0
    output_tokens: int = 0
    cache_creation_input_tokens: int = 0
    cache_read_input_tokens: int = 0

    def __add__(self, other: "TokenUsage") -> "TokenUsage":
        """Add two TokenUsage instances together."""
        return TokenUsage(
            input_tokens=self.input_tokens + other.input_tokens,
            output_tokens=self.output_tokens + other.output_tokens,
            cache_creation_input_tokens=self.cache_creation_input_tokens + other.cache_creation_input_tokens,
            cache_read_input_tokens=self.cache_read_input_tokens + other.cache_read_input_tokens
        )

    @property
    def total_input_tokens(self) -> int:
        """Total input tokens including cache tokens."""
        return self.input_tokens + self.cache_creation_input_tokens + self.cache_read_input_tokens

    @property
    def cache_efficiency_percentage(self) -> float:
        """Calculate cache efficiency as percentage of cache reads."""
        total_cacheable = self.cache_creation_input_tokens + self.cache_read_input_tokens
        if total_cacheable == 0:
            return 0.0
        return (self.cache_read_input_tokens / total_cacheable) * 100


@dataclass
class SessionMessage:
    """Represents a single message in a session."""

    role: str
    timestamp: str
    session_id: str
    cwd: Optional[str] = None
    usage: Optional[TokenUsage] = None
    model: Optional[str] = None

    @property
    def datetime(self) -> Optional[datetime]:
        """Get datetime from ISO timestamp.

        Returns:
            datetime object or None if timestamp is invalid/empty
        """
        if not self.timestamp or not self.timestamp.strip():
            return None
        try:
            return datetime.fromisoformat(self.timestamp.replace('Z', '+00:00'))
        except (ValueError, AttributeError):
            return None

    @classmethod
    def from_dict(cls, data: dict) -> "SessionMessage":
        """Create SessionMessage from parsed JSON dict."""
        # Check for nested message structure (newer format)
        message = data.get("message", {})

        # Try to get usage from nested message first, then fall back to top level
        usage_data = message.get("usage") or data.get("usage")
        usage = None
        if usage_data:
            usage = TokenUsage(
                input_tokens=usage_data.get("input_tokens", 0),
                output_tokens=usage_data.get("output_tokens", 0),
                cache_creation_input_tokens=usage_data.get("cache_creation_input_tokens", 0),
                cache_read_input_tokens=usage_data.get("cache_read_input_tokens", 0)
            )

        # Get role from message or top level
        role = message.get("role") or data.get("role", "")

        # Get model from message or top level
        model = message.get("model") or data.get("model")

        return cls(
            role=role,
            timestamp=data.get("timestamp", ""),
            session_id=data.get("sessionId", ""),
            cwd=data.get("cwd"),
            usage=usage,
            model=model
        )


@dataclass
class SessionStats:
    """Aggregated statistics for session(s)."""

    total_tokens: TokenUsage = field(default_factory=TokenUsage)
    message_count: int = 0
    session_ids: set[str] = field(default_factory=set)
    projects: set[str] = field(default_factory=set)
    earliest_timestamp: Optional[datetime] = None
    latest_timestamp: Optional[datetime] = None
    model_usage: dict[str, TokenUsage] = field(default_factory=dict)  # Tokens per model

    def add_message(self, message: SessionMessage):
        """Add a message to the statistics."""
        if message.usage:
            self.total_tokens += message.usage

            # Track tokens per model
            if message.model:
                if message.model not in self.model_usage:
                    self.model_usage[message.model] = TokenUsage()
                self.model_usage[message.model] += message.usage

        self.message_count += 1
        self.session_ids.add(message.session_id)

        if message.cwd:
            self.projects.add(message.cwd)

        # Only update timestamps if we have a valid datetime
        msg_dt = message.datetime
        if msg_dt is not None:
            if self.earliest_timestamp is None or msg_dt < self.earliest_timestamp:
                self.earliest_timestamp = msg_dt
            if self.latest_timestamp is None or msg_dt > self.latest_timestamp:
                self.latest_timestamp = msg_dt

    @property
    def session_count(self) -> int:
        """Number of unique sessions."""
        return len(self.session_ids)

    @property
    def project_count(self) -> int:
        """Number of unique projects."""
        return len(self.projects)


class SessionParser:
    """Parser for Claude Code session JSONL files."""

    def __init__(self, session_files: list[Path]):
        """
        Initialize session parser.

        Args:
            session_files: List of session JSONL file paths
        """
        self.session_files = session_files

    def parse_file(
        self,
        session_file: Path,
        time_filter: Optional[TimeFilter] = None
    ) -> Iterator[SessionMessage]:
        """
        Parse a single session file and yield messages.

        Args:
            session_file: Path to session JSONL file
            time_filter: Optional time filter

        Yields:
            SessionMessage instances
        """
        if not session_file.exists():
            return

        with open(session_file, 'r', encoding='utf-8') as f:
            for line in f:
                line = line.strip()
                if not line:
                    continue

                try:
                    data = json.loads(line)
                    message = SessionMessage.from_dict(data)

                    # Skip messages with invalid timestamps
                    if not message.timestamp or not message.timestamp.strip():
                        continue

                    # Apply time filter
                    if time_filter and not time_filter.matches_iso_string(message.timestamp):
                        continue

                    yield message

                except (json.JSONDecodeError, ValueError):
                    # Skip malformed lines
                    continue

    def parse_all(
        self,
        time_filter: Optional[TimeFilter] = None,
        project_filter: Optional[str] = None
    ) -> Iterator[SessionMessage]:
        """
        Parse all session files and yield messages.

        Args:
            time_filter: Optional time filter
            project_filter: Optional project path to filter by

        Yields:
            SessionMessage instances
        """
        for session_file in self.session_files:
            for message in self.parse_file(session_file, time_filter=time_filter):
                # Apply project filter
                if project_filter and message.cwd != project_filter:
                    continue

                yield message

    def get_stats(
        self,
        time_filter: Optional[TimeFilter] = None,
        project_filter: Optional[str] = None
    ) -> SessionStats:
        """
        Get aggregated statistics from all sessions.

        Args:
            time_filter: Optional time filter
            project_filter: Optional project path to filter by

        Returns:
            SessionStats with aggregated data
        """
        stats = SessionStats()

        for message in self.parse_all(time_filter=time_filter, project_filter=project_filter):
            stats.add_message(message)

        return stats

    def get_project_stats(
        self,
        time_filter: Optional[TimeFilter] = None
    ) -> dict[str, SessionStats]:
        """
        Get per-project statistics.

        Args:
            time_filter: Optional time filter

        Returns:
            Dict mapping project paths to SessionStats
        """
        project_stats: dict[str, SessionStats] = {}

        for message in self.parse_all(time_filter=time_filter):
            if not message.cwd:
                continue

            if message.cwd not in project_stats:
                project_stats[message.cwd] = SessionStats()

            project_stats[message.cwd].add_message(message)

        # Sort by total tokens (descending)
        return dict(
            sorted(
                project_stats.items(),
                key=lambda x: x[1].total_tokens.total_input_tokens + x[1].total_tokens.output_tokens,
                reverse=True
            )
        )

    def get_daily_stats(
        self,
        days: int = 7,
        time_filter: Optional[TimeFilter] = None
    ) -> dict[str, SessionStats]:
        """
        Get token statistics aggregated by day.

        Args:
            days: Number of days to include (default 7), including today
            time_filter: Optional time filter

        Returns:
            Dict mapping date strings (YYYY-MM-DD) to SessionStats
        """
        from datetime import timedelta

        # Initialize dict for each day
        daily_stats: dict[str, SessionStats] = {}
        today = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

        for i in range(days - 1, -1, -1):  # days entries, including today
            day = today - timedelta(days=i)
            date_key = day.strftime('%Y-%m-%d')
            daily_stats[date_key] = SessionStats()

        # Parse messages and bucket by day
        for message in self.parse_all(time_filter=time_filter):
            msg_dt = message.datetime
            if msg_dt:
                date_key = msg_dt.strftime('%Y-%m-%d')
                if date_key in daily_stats:
                    daily_stats[date_key].add_message(message)

        return daily_stats

    def get_daily_cost_trend(
        self,
        days: int = 7,
        time_filter: Optional[TimeFilter] = None
    ) -> dict[str, dict[str, float]]:
        """
        Get daily cost statistics for trend visualization.

        Args:
            days: Number of days to include (default 7), including today
            time_filter: Optional time filter

        Returns:
            Dict mapping date strings (YYYY-MM-DD) to cost breakdowns with keys:
            - input_cost: Cost of input tokens
            - output_cost: Cost of output tokens
            - cache_write_cost: Cost of cache creation
            - cache_read_cost: Cost of cache reads
            - total_cost: Total cost for the day
        """
        # Define pricing constants locally to avoid cyclic import
        DEFAULT_PRICING = {
            'input_per_mtok': 3.0,
            'output_per_mtok': 15.0,
            'cache_write_per_mtok': 3.75,
            'cache_read_per_mtok': 0.30,
        }

        daily_stats = self.get_daily_stats(days=days, time_filter=time_filter)

        daily_costs = {}
        for date_str, stats in daily_stats.items():
            # Calculate costs for each token type using default pricing
            # (This is an approximation since model-specific pricing varies)
            input_cost = (stats.total_tokens.input_tokens / 1_000_000) * DEFAULT_PRICING['input_per_mtok']
            output_cost = (stats.total_tokens.output_tokens / 1_000_000) * DEFAULT_PRICING['output_per_mtok']
            cache_write_cost = (stats.total_tokens.cache_creation_input_tokens / 1_000_000) * DEFAULT_PRICING['cache_write_per_mtok']
            cache_read_cost = (stats.total_tokens.cache_read_input_tokens / 1_000_000) * DEFAULT_PRICING['cache_read_per_mtok']

            daily_costs[date_str] = {
                'input_cost': round(input_cost, 4),
                'output_cost': round(output_cost, 4),
                'cache_write_cost': round(cache_write_cost, 4),
                'cache_read_cost': round(cache_read_cost, 4),
                'total_cost': round(input_cost + output_cost + cache_write_cost + cache_read_cost, 4),
            }

        return daily_costs

    def get_project_daily_stats(
        self,
        days: int = 7,
        time_filter: Optional[TimeFilter] = None,
        max_projects: int = 10
    ) -> dict[str, dict[str, dict[str, float]]]:
        """
        Get daily cost statistics per project for trend visualization.

        Args:
            days: Number of days to include (default 7), including today
            time_filter: Optional time filter
            max_projects: Maximum number of top projects to include

        Returns:
            Dict mapping project names to date strings to cost data:
            {
                "project_name": {
                    "2024-01-01": {"total_cost": 0.50, ...},
                    ...
                }
            }
        """
        from datetime import timedelta

        # Define pricing constants locally to avoid cyclic import
        DEFAULT_PRICING = {
            'input_per_mtok': 3.0,
            'output_per_mtok': 15.0,
            'cache_write_per_mtok': 3.75,
            'cache_read_per_mtok': 0.30,
        }

        # First, get overall project stats to find top projects by total cost
        all_project_stats = self.get_project_stats(time_filter=time_filter)

        # Calculate total cost for each project
        project_costs = {}
        for project_path, stats in all_project_stats.items():
            input_cost = (stats.total_tokens.input_tokens / 1_000_000) * DEFAULT_PRICING['input_per_mtok']
            output_cost = (stats.total_tokens.output_tokens / 1_000_000) * DEFAULT_PRICING['output_per_mtok']
            cache_write_cost = (stats.total_tokens.cache_creation_input_tokens / 1_000_000) * DEFAULT_PRICING['cache_write_per_mtok']
            cache_read_cost = (stats.total_tokens.cache_read_input_tokens / 1_000_000) * DEFAULT_PRICING['cache_read_per_mtok']
            project_costs[project_path] = input_cost + output_cost + cache_write_cost + cache_read_cost

        # Get top projects
        top_projects = sorted(project_costs.items(), key=lambda x: x[1], reverse=True)[:max_projects]

        # Initialize result structure
        result = {}
        today = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

        # Initialize dates
        date_keys = []
        for i in range(days - 1, -1, -1):
            day = today - timedelta(days=i)
            date_keys.append(day.strftime('%Y-%m-%d'))

        # Collect top project paths for efficient lookup
        project_paths = [project_path for project_path, _ in top_projects]
        project_paths_set = set(project_paths)

        # Prepare per-project, per-day stats for all top projects in a single pass
        project_stats_map: dict[str, dict[str, SessionStats]] = {}
        for project_path in project_paths:
            project_stats_map[project_path] = {date_key: SessionStats() for date_key in date_keys}

        # Single pass over all messages, grouped by project and date (O(n) complexity)
        for message in self.parse_all(time_filter=time_filter):
            project_path = message.cwd
            if project_path not in project_paths_set:
                continue

            msg_dt = message.datetime
            if not msg_dt:
                continue

            date_key = msg_dt.strftime('%Y-%m-%d')
            project_stats = project_stats_map.get(project_path)
            if project_stats and date_key in project_stats:
                project_stats[date_key].add_message(message)

        # For each top project, convert collected stats to daily costs
        for project_path, _ in top_projects:
            # Get short project name for display (works for both Unix and Windows paths)
            project_name = Path(project_path).name

            project_stats = project_stats_map.get(project_path, {})

            # Convert to costs
            daily_costs = {}
            for date_key, stats in project_stats.items():
                input_cost = (stats.total_tokens.input_tokens / 1_000_000) * DEFAULT_PRICING['input_per_mtok']
                output_cost = (stats.total_tokens.output_tokens / 1_000_000) * DEFAULT_PRICING['output_per_mtok']
                cache_write_cost = (stats.total_tokens.cache_creation_input_tokens / 1_000_000) * DEFAULT_PRICING['cache_write_per_mtok']
                cache_read_cost = (stats.total_tokens.cache_read_input_tokens / 1_000_000) * DEFAULT_PRICING['cache_read_per_mtok']

                daily_costs[date_key] = {
                    'input_cost': round(input_cost, 4),
                    'output_cost': round(output_cost, 4),
                    'cache_write_cost': round(cache_write_cost, 4),
                    'cache_read_cost': round(cache_read_cost, 4),
                    'total_cost': round(input_cost + output_cost + cache_write_cost + cache_read_cost, 4),
                }

            result[project_name] = daily_costs

        return result



"""Utilities for filtering data by time ranges."""

from datetime import datetime, timedelta
from typing import Optional
from dateutil import parser as date_parser


class TimeFilter:
    """Handles time-based filtering for usage data."""

    def __init__(
        self,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None
    ):
        """
        Initialize time filter.

        Args:
            start_time: Start of time range (inclusive)
            end_time: End of time range (inclusive)
        """
        self.start_time = start_time
        self.end_time = end_time or datetime.now()

    @classmethod
    def from_preset(cls, preset: str) -> "TimeFilter":
        """
        Create a TimeFilter from a preset string.

        Args:
            preset: One of 'today', 'week', 'month', 'quarter', 'year', or 'all'

        Returns:
            TimeFilter instance
        """
        now = datetime.now()
        today_start = now.replace(hour=0, minute=0, second=0, microsecond=0)

        if preset == "today":
            return cls(start_time=today_start, end_time=now)
        elif preset == "week":
            week_start = today_start - timedelta(days=7)
            return cls(start_time=week_start, end_time=now)
        elif preset == "month":
            month_start = today_start - timedelta(days=30)
            return cls(start_time=month_start, end_time=now)
        elif preset == "quarter":
            # Calculate start of current quarter
            # Q1: Jan-Mar (1-3), Q2: Apr-Jun (4-6), Q3: Jul-Sep (7-9), Q4: Oct-Dec (10-12)
            current_month = now.month
            quarter_start_month = ((current_month - 1) // 3) * 3 + 1
            quarter_start = now.replace(month=quarter_start_month, day=1, hour=0, minute=0, second=0, microsecond=0)
            return cls(start_time=quarter_start, end_time=now)
        elif preset == "year":
            year_start = today_start - timedelta(days=365)
            return cls(start_time=year_start, end_time=now)
        elif preset == "all":
            return cls(start_time=None, end_time=now)
        else:
            raise ValueError(f"Unknown preset: {preset}")

    @classmethod
    def from_since(cls, since_str: str) -> "TimeFilter":
        """
        Create a TimeFilter from a 'since' date string.

        Args:
            since_str: Date string in various formats (YYYY-MM-DD, etc.)

        Returns:
            TimeFilter instance
        """
        start_time = date_parser.parse(since_str)
        return cls(start_time=start_time, end_time=datetime.now())

    def matches_timestamp_ms(self, timestamp_ms: int) -> bool:
        """
        Check if a millisecond timestamp falls within the filter range.

        Args:
            timestamp_ms: Timestamp in milliseconds since epoch

        Returns:
            True if timestamp is within range
        """
        dt = datetime.fromtimestamp(timestamp_ms / 1000)
        return self.matches_datetime(dt)

    def matches_datetime(self, dt: datetime) -> bool:
        """
        Check if a datetime falls within the filter range.

        Args:
            dt: Datetime to check

        Returns:
            True if datetime is within range
        """
        # Convert to naive datetime for comparison (remove timezone info)
        if dt.tzinfo is not None:
            dt = dt.replace(tzinfo=None)

        if self.start_time and dt < self.start_time:
            return False
        if self.end_time and dt > self.end_time:
            return False
        return True

    def matches_iso_string(self, iso_str: str) -> bool:
        """
        Check if an ISO format timestamp string falls within the filter range.

        Args:
            iso_str: ISO format timestamp string

        Returns:
            True if timestamp is within range
        """
        dt = datetime.fromisoformat(iso_str.replace('Z', '+00:00'))
        return self.matches_datetime(dt)

    def get_description(self) -> str:
        """Get a human-readable description of the time range."""
        if self.start_time is None:
            return "All time"

        now = datetime.now()
        delta = now - self.start_time

        # Check if it's current quarter
        current_month = now.month
        quarter_start_month = ((current_month - 1) // 3) * 3 + 1
        quarter_start = now.replace(month=quarter_start_month, day=1, hour=0, minute=0, second=0, microsecond=0)

        if self.start_time == quarter_start:
            quarter_num = (quarter_start_month - 1) // 3 + 1
            return f"Q{quarter_num} {now.year}"
        elif delta.days == 0:
            return "Today"
        elif delta.days == 7:
            return "Last 7 days"
        elif delta.days == 30:
            return "Last 30 days"
        elif delta.days == 365:
            return "Last year"
        else:
            return f"Since {self.start_time.strftime('%Y-%m-%d')}"



"""Flask application for Claude Monitor web interface.

This module provides the Flask app factory for creating and configuring
the Claude Monitor web application.
"""

import os
from datetime import datetime
from flask import Flask, jsonify
from typing import Optional

from ..utils.paths import ClaudeDataPaths, get_claude_paths
from .services.dashboard_service import DashboardService


def create_app(claude_data_paths: Optional[ClaudeDataPaths] = None) -> Flask:
    """
    Create and configure Flask application.

    Args:
        claude_data_paths: Optional ClaudeDataPaths instance. If not provided,
                         uses default Claude data directory.

    Returns:
        Configured Flask application
    """
    # Get the web module directory for template and static paths
    web_dir = os.path.dirname(os.path.abspath(__file__))
    template_dir = os.path.join(web_dir, 'templates')
    static_dir = os.path.join(web_dir, 'static')

    # Create Flask app with explicit template and static directories
    app = Flask(
        __name__,
        template_folder=template_dir,
        static_folder=static_dir,
        static_url_path='/static'
    )

    # Configuration
    app.config['JSON_SORT_KEYS'] = False

    # Jinja2 configuration
    app.jinja_env.auto_reload = True
    app.jinja_env.cache = None if app.debug else {}

    # Add template globals for use in templates
    app.jinja_env.globals.update(
        now=datetime.now,
        str=str,
        len=len,
        enumerate=enumerate,
    )

    # Add custom Jinja2 filters
    @app.template_filter('format_number')
    def format_number(value: float, decimals: int = 2) -> str:
        """Format a number with specified decimal places."""
        if value is None:
            return '0'
        try:
            return f'{float(value):.{decimals}f}'
        except (ValueError, TypeError):
            return str(value)

    @app.template_filter('format_percent')
    def format_percent(value: float, decimals: int = 1) -> str:
        """Format a value as a percentage."""
        if value is None:
            return '0%'
        try:
            return f'{float(value):.{decimals}f}%'
        except (ValueError, TypeError):
            return str(value)

    @app.template_filter('format_time')
    def format_time(seconds: float) -> str:
        """Format seconds into a human-readable time string."""
        if seconds is None or seconds < 0:
            return '0s'

        if seconds < 60:
            return f'{int(seconds)}s'
        elif seconds < 3600:
            minutes = seconds / 60
            return f'{minutes:.1f}m'
        else:
            hours = seconds / 3600
            return f'{hours:.1f}h'

    @app.template_filter('format_compact')
    def format_compact(value: float) -> str:
        """Format a large number in compact form (e.g., 1.5M, 234K)."""
        if value is None:
            return '0'
        try:
            num = float(value)
            if num >= 1_000_000_000:
                return f'{num / 1_000_000_000:,.2f}B'
            elif num >= 1_000_000:
                return f'{num / 1_000_000:,.2f}M'
            elif num >= 1_000:
                return f'{num / 1_000:,.1f}K'
            else:
                return f'{num:,.0f}'
        except (ValueError, TypeError):
            return str(value)

    # Initialize dashboard service
    if claude_data_paths is None:
        claude_data_paths = get_claude_paths()

    dashboard_service = DashboardService(claude_data_paths)

    # Store service in app context for routes to access
    app.dashboard_service = dashboard_service

    # Register blueprints (if they exist)
    # These will be imported from routes submodule
    try:
        from .routes.dashboard import dashboard_bp
        app.register_blueprint(dashboard_bp)
    except (ImportError, ModuleNotFoundError):
        # Dashboard blueprint is optional
        pass

    try:
        from .routes.api import api_bp
        app.register_blueprint(api_bp)
    except (ImportError, ModuleNotFoundError):
        # API blueprint is optional
        pass

    try:
        from .routes.charts import charts_bp
        app.register_blueprint(charts_bp)
    except (ImportError, ModuleNotFoundError):
        # Charts blueprint is optional
        pass

    # Error handlers
    @app.errorhandler(404)
    def not_found(error):
        """Handle 404 Not Found errors."""
        return jsonify({
            'error': 'Not Found',
            'message': 'The requested resource was not found.',
            'status': 404
        }), 404

    @app.errorhandler(500)
    def internal_error(error):
        """Handle 500 Internal Server errors."""
        return jsonify({
            'error': 'Internal Server Error',
            'message': 'An unexpected error occurred. Please try again later.',
            'status': 500
        }), 500

    @app.errorhandler(400)
    def bad_request(error):
        """Handle 400 Bad Request errors."""
        return jsonify({
            'error': 'Bad Request',
            'message': 'The request could not be processed.',
            'status': 400
        }), 400

    # Add health check endpoint
    @app.route('/health')
    def health():
        """Health check endpoint."""
        return jsonify({'status': 'ok', 'timestamp': datetime.now().isoformat()}), 200

    # Default route
    @app.route('/')
    def index():
        """Index route - display welcome message."""
        return jsonify({
            'message': 'Claude Monitor Web Server',
            'version': '1.0.0',
            'status': 'running',
            'timestamp': datetime.now().isoformat()
        }), 200

    # Add CORS headers for development (if needed)
    @app.after_request
    def add_headers(response):
        """Add headers to response for development."""
        # Allow same-origin requests by default
        response.headers['X-Content-Type-Options'] = 'nosniff'
        response.headers['X-Frame-Options'] = 'SAMEORIGIN'
        response.headers['X-XSS-Protection'] = '1; mode=block'

        # In development, add CORS headers
        if app.debug:
            response.headers['Access-Control-Allow-Origin'] = '*'
            response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, DELETE, OPTIONS'
            response.headers['Access-Control-Allow-Headers'] = 'Content-Type'

        return response

    return app



"""Setup file for claude-monitor."""

from setuptools import setup, find_packages

setup(
    name="claude-monitor",
    version="1.0.1",
    description="Web-based usage monitoring tool for Claude Code",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    package_data={
        "claude_monitor.web": [
            "templates/**/*.html",
            "static/**/*",
        ],
    },
    include_package_data=True,
    install_requires=[
        "click>=8.1.0",
        "rich>=13.0.0",
        "python-dateutil>=2.8.0",
        "Flask>=3.0.0",
        "Jinja2>=3.1.0",
    ],
    entry_points={
        "console_scripts": [
            "claude-monitor=claude_monitor.cli:main",
        ],
    },
    python_requires=">=3.9",
)
